<?php

//=================================================
// サブメニュー クーポンコード入力画面
//=================================================

require_once(dirname(__FILE__) . '/../util.php');
require_once(get_plugin_root() . '/config.php');
require_once(get_plugin_root() . '/api/user_certification.php');
require_once(get_plugin_root() . '/db/operation.php');

function calliope_coupon_code_input_page()
{
  $form_items = CalliopeConfig::CALLIOPE_COUPON_CODE_INPUT_FORM_ITEMS;

  $result = [];
  $errors = [];
  $expired_date = '';

  $uc = new UserCertification();

  if (get_post_value('posted') == 'save') {

    $errors = calliope_coupon_code_input_page_validation();

    if (empty($errors)) {
      $result = $uc->coupon_code_registration(get_post_value('coupon_code'));

      if ($result['code'] == ApiBase::SUCCESS) {
        // DBにuidを登録
        $cols = [
          'is_coupon_code_registration' => 1,
        ];

        DBoperation::db_update(CalliopeConfig::TABLE_CALLIOPE_USER, $cols, ['id' => 1]);
      }
    }
  }

  $_exrs = $uc->get_expired_date();
  if ($_exrs['code'] == ApiBase::SUCCESS) {
    $expired_date = date('Y/m/d', strtotime($_exrs['data']->expiredDate));
  }

  ///// jsの読み込み
  wp_enqueue_script('calliope-scripts', get_plugin_root_uri('js/calliope_script.js'));

?>

  <?php calliope_wrap_head(); ?>

  <?php
  $now = date('Y/m/d');
  if (!empty($expired_date) && $now > $expired_date) :
  ?>
    <div class="error">
      <p><strong>クーポンの有効期限が切れています。</strong></p>
    </div>
  <?php else : ?>
    <?php foreach ($errors as $error) : ?>
      <div class="error">
        <p><strong><?php echo $error['label'] ?>は<?php echo $error['message'] ?></strong></p>
      </div>
    <?php endforeach; ?>

    <?php if (!empty($result) && $result['code'] == ApiBase::ERROR) : ?>
      <div class="error">
        <p><strong><?php echo $result['message'] ?></strong></p>
      </div>
    <?php endif; ?>

    <?php if (!empty($result) && $result['code'] == ApiBase::SUCCESS) : ?>
      <div class="updated">
        <p><strong>クーポンコードの認証が完了しました</strong></p>
        <p><strong>有効期限は<?php echo $expired_date ?>です</strong></p>
      </div>
      <div class="to-main-menu">
        <a href="<?php echo admin_url('admin.php?page=calliope_menu') ?>">メイン設定に進む →</a>
      </div>
    <?php endif; ?>

  <?php endif; ?>

  <form method="post" action="<?php echo str_replace('%7E', '~', $_SERVER['REQUEST_URI']); ?>">
    <div class="calliope_coupon_code_input_page">
      <div class="calliope-area">
        <div class="calliope-wrapper">
          <div class="calliope-header">
            <h1>クーポンコード入力</h1>
          </div>
          <div class="calliope-contents-area">
            <div class="calliope-contents-wrapper">
              <div class="input-wrapper">
                <div class="input-header">
                  <div class="input-label">クーポンコード</div>
                  <div class="input-hint">
                    <div class="input-hint-label"><img src="<?php echo CalliopeConfig::get_img_uri(); ?>hint_icon.png"></div>
                    <div class="input-hint-text">クーポンコードを入力してください</div>
                  </div>
                </div>
                <input type="text" name="coupon_code" value="<?php echo $form_items['coupon_code']['val'] ?>">
              </div>
              <div class="bottom-action-wrapper">
                <input type="hidden" name="posted" value="save">
                <button type="submit" name="submit" class="submit-btn button button-primary" value="save">クーポンコードを登録する</button>
              </div>
            </div>
          </div>
        </div>
      </div>
    </div>
  </form>

  <?php calliope_wrap_tail(); ?>

<?php
}

function calliope_coupon_code_input_page_validation()
{
  $errors = [];
  $form_items = CalliopeConfig::CALLIOPE_COUPON_CODE_INPUT_FORM_ITEMS;

  foreach ($form_items as $key => $form_item) {
    $form_items[$key] = get_post_value($key);
    if ($form_item['required'] == true && (get_post_value($key) == null || get_post_value($key) == '')) {
      $errors[] = [
        'label' => $form_item['label'],
        'message' => '必須項目です',
      ];
    }
  }

  return $errors;
}
