<?php

//=================================================
// サブメニュー ダッシュボード
//=================================================

require_once(dirname(__FILE__) . '/../util.php');
require_once(get_plugin_root() . '/config.php');
require_once(get_plugin_root() . '/db/operation.php');
require_once(get_plugin_root() . '/api/stripe.php');
require_once(get_plugin_root() . '/api/user_certification.php');

function calliope_dashboard_page()
{
  global $wpdb;

  /////// 全期間の投稿件数
  $all_period_active_post_count = get_all_period_active_post_count();

  /////// 今月の投稿件数
  $monthly_active_post_count = get_monthly_active_post_count();

  /////// メイン設定
  $setting_form_items = CalliopeConfig::CALLIOPE_SETTING_FORM_ITEMS;
  $calliope_setting = $wpdb->get_row("SELECT * FROM " . CalliopeConfig::TABLE_CALLIOPE_SETTING . " WHERE id = 1");
  if ($calliope_setting != null) {
    foreach ($setting_form_items as $key => $form_item) {
      $setting_form_items[$key]['val'] = $calliope_setting->{$key};
    }
  }

  /////// 直近の投稿
  $np_calliope_feeds_schedules = $wpdb->get_results(
    "SELECT * FROM " . CalliopeConfig::TABLE_CALLIOPE_FEEDS_SCHEDULE . "
    WHERE status = 'done'
    ORDER BY `created_at` desc
    LIMIT 5"
  );

  /////// メールアドレス
  $calliope_user = $wpdb->get_row("SELECT * FROM " . CalliopeConfig::TABLE_CALLIOPE_USER . " WHERE id = 1");
  $email = $calliope_user->mail;

  /////// 記事制限
  $post_limit = get_post_limit();

  /////// サブスク登録プラン
  $subsc_plan = '未登録';
  $stripe = new Stripe();
  $s_result = $stripe->check_subscription_status($email);

  if ($s_result['code'] == ApiBase::SUCCESS) {
    if ($s_result['data']->status == "active") {
      $subsc_plan = $s_result['data']->product->name;
    }
  }

  /////// クーポンの有効期限
  $coupon_expired_date = '未登録';

  // 有効期限を取得する
  if (check_is_coupon_code_registration()) {
    $uc = new UserCertification();
    $_exrs = $uc->get_expired_date();
    if ($_exrs['code'] == ApiBase::SUCCESS) {
      $coupon_expired_date = $_exrs['data']->expiredDate;
    }
  }
  // サブスク登録済みの場合は未登録にする
  if ($s_result['code'] == ApiBase::SUCCESS) {
    if ($s_result['data']->status == "active") {
      $coupon_expired_date = '未登録';
    }
  }


  ///// jsの読み込み
  wp_enqueue_script('calliope-scripts', get_plugin_root_uri('js/calliope_script.js'));

?>

  <?php calliope_wrap_head(); ?>

  <div class="calliope_dashboard_page">
    <div class="calliope-area">
      <div class="calliope-wrapper">
        <div class="calliope-header">
          <h1>ダッシュボード</h1>
        </div>
        <div class="calliope-contents-area">
          <div class="dashboard-head-area">
            <div class="dashboard-post-count-area dashboard-content-wrapper">
              <div class="dashboard-content-title">データ</div>
              <div class="margin-top-20"><span class="devider"></span></div>
              <div class="dashboard-post-count-title margin-top-20 margin-left-10">投稿件数</div>
              <div class="dashboard-ap-post-count margin-top-20 margin-left-10"><?php echo $all_period_active_post_count ?> 件</div>
              <div class="margin-top-30"><span class="devider"></span></div>
              <div class="dashboard-post-count-title margin-top-20 margin-left-10">今月の投稿件数</div>
              <div class="dashboard-mp-post-count margin-top-20 margin-left-10 margin-bottom-20"><?php echo $monthly_active_post_count ?> 件</div>
            </div>
            <div class="dashboard-setting-area dashboard-content-wrapper">
              <div class="dashboard-content-title">基本設定</div>
              <div class="margin-top-20"><span class="devider"></span></div>
              <div class="dashboard-setting-content margin-top-40">
                <div class="dashboard-setting-title">OpenAI APIキー</div>
                <div class="dashboard-setting-value"><?php echo !empty($setting_form_items['open_ai_api_key']['val']) ? '設定済み' : '未設定' ?></div>
              </div>
              <div class="dashboard-setting-content margin-top-40 margin-bottom-20">
                <div class="dashboard-setting-title">自動投稿</div>
                <div class="dashboard-setting-value"><?php echo CalliopeConfig::CALLIOPE_IS_CRON_EXEC[$setting_form_items['is_cron_exec']['val']] ?></div>
              </div>
              <div class="margin-top-40"><span class="devider"></span></div>
              <div class="dashboard-main-setting-link margin-top-10">
                <a href="<?php echo admin_url('admin.php?page=calliope_setting') ?>">メイン設定に進む →</a>
              </div>
            </div>
            <div class="dashboard-link-area">
              <div class="dashboard-manual-link-area dashboard-content-wrapper dashboard-link-area">
                <div class="dashboard-manual-link">
                  <a href="<?php echo admin_url('admin.php?page=calliope_manual') ?>">
                    <div class="dashboard-link-block">
                      <div class="dashboard-link-block-img"><img src="<?php echo CalliopeConfig::get_img_uri(); ?>dashboard/manual.png"></div>
                      <div class="dashboard-link-block-label">マニュアルはこちら</div>
                    </div>
                  </a>
                </div>
              </div>
              <div class="dashboard-subsc-link-area dashboard-content-wrapper dashboard-link-area">
                <a href="<?php echo admin_url('admin.php?page=calliope_subsc_purchase') ?>">
                  <div class="dashboard-link-block">
                    <div class="dashboard-link-block-img"><img src="<?php echo CalliopeConfig::get_img_uri(); ?>dashboard/subsc.png"></div>
                    <div class="dashboard-link-block-label">サブスクリプションの登録はこちら</div>
                  </div>
                </a>
              </div>
              <div class="dashboard-feed-link-area dashboard-content-wrapper">
                <a href="<?php echo admin_url('admin.php?page=calliope_feeds') ?>">
                  <div class="dashboard-link-block">
                    <div class="dashboard-link-block-img"><img src="<?php echo CalliopeConfig::get_img_uri(); ?>dashboard/feed.png"></div>
                    <div class="dashboard-link-block-label">フィード設定に進む</div>
                  </div>
                </a>
              </div>
            </div>
          </div>
          <div class="dashboard-near-post-area dashboard-content-wrapper">
            <div class="dashboard-content-title">直近の投稿</div>
            <?php foreach ($np_calliope_feeds_schedules as $np_calliope_feeds_schedule) : ?>
              <div class="dashboard-near-post-row margin-top-40">
                <div class="dashboard-near-post-row-left-area">
                  <div class="dashboard-near-post-title"><?php echo $np_calliope_feeds_schedule->title ?></div>
                  <div class="dashboard-near-post-link-wrapper margin-top-20">
                    <div class="dashboard-near-post-link-post">
                      <a href="<?php echo admin_url('post.php?post=' . $np_calliope_feeds_schedule->post_id . '&action=edit') ?>">記事設定に移動</a>
                    </div>
                    <div class="dashboard-near-post-link-feed margin-left-20">
                      <a href="<?php echo admin_url('admin.php?page=calliope_feed_edit&action=edit&id=' . $np_calliope_feeds_schedule->calliope_feeds_id) ?>">フィード設定に移動</a>
                    </div>
                  </div>
                </div>
                <div class="dashboard-near-post-row-right-area">
                  <div class="dashboard-near-post-date"><?php echo $np_calliope_feeds_schedule->created_at ?></div>
                </div>
              </div>
              <div class="margin-top-20"><span class="devider"></span></div>
            <?php endforeach; ?>
          </div>
          <div class="dashboard-user-info-area dashboard-content-wrapper margin-top-30">
            <div class="dashboard-content-title">お客様情報</div>
            <div class="margin-top-20"><span class="devider"></span></div>
            <div class="dashboard-user-info-content margin-top-40">
              <div class="dashboard-user-info-title">メールアドレス</div>
              <div class="dashboard-user-info-value"><?php echo $email ?></div>
            </div>
            <div class="dashboard-user-info-content margin-top-40">
              <div class="dashboard-user-info-title">今月の生成記事数</div>
              <div class="dashboard-user-info-value">
                <?php echo $monthly_active_post_count ?>件 /
                <?php if (!empty($post_limit)) : ?>
                  <?php echo $post_limit ?>件中
                <?php else : ?>
                  無制限
                <?php endif; ?>
              </div>
            </div>
            <div class="dashboard-user-info-content margin-top-40">
              <div class="dashboard-user-info-title">サブスク登録プラン</div>
              <div class="dashboard-user-info-value"><?php echo $subsc_plan ?></div>
            </div>
            <div class="dashboard-user-info-content margin-top-40 margin-bottom-20">
              <div class="dashboard-user-info-title">クーポンの有効期限</div>
              <div class="dashboard-user-info-value"><?php echo $coupon_expired_date ?></div>
            </div>
          </div>
        </div>
      </div>
    </div>
  </div>

  <?php calliope_wrap_tail(); ?>

<?php
}
