<?php

//=================================================
// サブメニュー ページ内容の表示・更新処理
//=================================================

require_once(dirname(__FILE__) . '/../util.php');
require_once(get_plugin_root() . '/calliope.php');
require_once(get_plugin_root() . '/calliope_logger.php');
require_once(get_plugin_root() . '/scheduler.php');
require_once(get_plugin_root() . '/config.php');
require_once(get_plugin_root() . '/db/operation.php');

function calliope_feed_edit_page()
{
  global $wpdb;
  $cfs_id = 0;

  $id = get_get_value('id') != null ? get_get_value('id') : null;

  $form_items = CalliopeConfig::CALLIOPE_FEED_EDIT_FORM_ITEMS;
  $errors = [];

  if (get_post_value('submited') == 'submited') {
    //設定画面で入力された設定値を保存

    $id = get_post_value('id');

    $errors = calliope_feed_edit_page_validation();

    if (empty($errors)) {
      $cols = [];
      foreach ($form_items as $key => $form_item) {
        if ($key == 'category' || $key == 'tag') {
          $cols[$key] = is_array(get_post_value($key)) ? json_encode(get_post_value($key)) : json_encode([]);
        } else {
          $cols[$key] = get_post_value($key);
        }
      }

      if ($id == null || $id == '') {
        DBoperation::db_insert(CalliopeConfig::TABLE_CALLIOPE_FEEDS, $cols);

        $id = $wpdb->insert_id;
      } else {
        DBoperation::db_update(CalliopeConfig::TABLE_CALLIOPE_FEEDS, $cols, ['id' => $id]);
      }
    }
  }

  // 編集時の初期値を設定する
  if ($id != null) {
    $calliope_feed = $wpdb->get_row("SELECT * FROM " . CalliopeConfig::TABLE_CALLIOPE_FEEDS . " WHERE id = " . $id);
    if ($calliope_feed != null) {
      foreach ($form_items as $key => $form_item) {
        if ($key == 'category' || $key == 'tag') {
          $form_items[$key]['val'] = json_decode($calliope_feed->{$key});
        } else {
          $form_items[$key]['val'] = $calliope_feed->{$key};
        }
      }
    }
  }

  // 設定を確認する
  $calliope_setting = $wpdb->get_row("SELECT * FROM " . CalliopeConfig::TABLE_CALLIOPE_SETTING . " WHERE id = 1");

  // バリデーションエラー時の初期値を設定する
  if (!empty($errors)) {
    $form_items = item_posted_value_init_feed_edit_page();
  }

  ///// フラグを整理
  $is_posted_submit_save = get_post_value('submit-save') == 'save' && empty($errors) ? true : false;
  $is_posted_submit_test_autogenerator = get_post_value('submit-test-autogenerator') == 'test-autogenerator' && empty($errors) ? true : false;

  // 自動投稿をテスト実行する
  if ($is_posted_submit_test_autogenerator) {
    $cs = new CalliopeScheduler(true);
    $cfs_id = $cs->run($id);
  }

  ///// jsの読み込み
  wp_enqueue_script('calliope-scripts', get_plugin_root_uri('js/calliope_script.js'));
  wp_enqueue_script('calliope-scripts-calliope_feed_edit_page', get_plugin_root_uri('js/calliope_feed_edit_page.js'));
?>

  <?php calliope_wrap_head(); ?>

  <?php if ($is_posted_submit_save) : ?>
    <div class="updated">
      <p><strong>設定を保存しました</strong></p>
    </div>
  <?php endif; ?>

  <?php foreach ($errors as $error) : ?>
    <div class="error"><?php echo $error['label'] ?>は <?php echo $error['message'] ?></div>
  <?php endforeach; ?>

  <form method="post" name="form" action="<?php echo str_replace('%7E', '~', $_SERVER['REQUEST_URI']); ?>">
    <div class="calliope_feed_edit_page">
      <div class="calliope-area">
        <div class="calliope-wrapper">
          <div class="calliope-header">
            <h1>フィード編集</h1>
          </div>
          <div class="back-to-feeds-page-link"><a href="<?php echo admin_url('admin.php?page=calliope_feeds') ?>">← フィード設定に戻る</a></div>
          <div class="calliope-contents-area">
            <div class="calliope-contents-wrapper">
              <div class="input-index top">基本設定</div>
              <div class="input-wrapper">
                <div class="input-header">
                  <div class="input-label"><?php echo $form_items['name']['label'] ?></div>
                  <div class="input-required">*</div>
                  <div class="input-hint">
                    <div class="input-hint-label"><img src="<?php echo CalliopeConfig::get_img_uri(); ?>hint_icon.png"></div>
                    <div class="input-hint-text">
                      フィード設定で表示されるフィードの名前です。</br>
                      自動生成には影響ないのでわかりやすい名称をつけましょう。
                    </div>
                  </div>
                </div>
                <input type="text" name="name" id="input-name" value="<?php echo $form_items['name']['val'] ?>">
              </div>
              <div class="input-wrapper">
                <div class="input-header">
                  <div class="input-label"><?php echo $form_items['keyword']['label'] ?></div>
                  <div class="input-required">*</div>
                  <div class="input-hint">
                    <div class="input-hint-label"><img src="<?php echo CalliopeConfig::get_img_uri(); ?>hint_icon.png"></div>
                    <div class="input-hint-text">
                      投稿のキーワードを設定できます。自動投稿を実行するときに、ここで設定したキーワードに関する記事を生成します。</br>
                      「筋トレ」「睡眠」などの単語を設定しましょう。
                    </div>
                  </div>
                </div>
                <input type="text" name="keyword" id="input-keyword" value="<?php echo $form_items['keyword']['val'] ?>">
              </div>
              <div class="input-wrapper">
                <div class="input-header">
                  <div class="input-label"><?php echo $form_items['related_keywords']['label'] ?></div>
                  <div class="input-hint">
                    <div class="input-hint-label"><img src="<?php echo CalliopeConfig::get_img_uri(); ?>hint_icon.png"></div>
                    <div class="input-hint-text">
                      関連するキーワードを登録できます。改行で複数個のキーワードを登録できます。登録していない場合は自動で関連するキーワードを選定し、記事を作成します。</br>
                      複数個のキーワードを登録した場合、その中からランダムで選んだキーワードで記事を作成します。多くのキーワードを登録すると、記事にばらつきが生まれやすいです。
                    </div>
                  </div>
                </div>
                <div class="related-keyword-gen-link">
                  <div id="keyword-autogen-exec"><a href="javascript:void(0)">関連するキーワードを自動で生成する</a></div>
                  <div id="keyword-autogen-message"></div>
                </div>
                <textarea name="related_keywords" id="input-related_keywords" rows="9" cols="50"><?php echo $form_items['related_keywords']['val'] ?></textarea>
              </div>
              <div class="input-wrapper">
                <div class="input-header">
                  <div class="input-label"><?php echo $form_items['custom_prompt']['label'] ?></div>
                  <div class="input-hint">
                    <div class="input-hint-label"><img src="<?php echo CalliopeConfig::get_img_uri(); ?>hint_icon.png"></div>
                    <div class="input-hint-text">
                      カスタムプロンプトを登録できます。記事生成時に、個々で設定したプロンプトを元に内容生成を行います。</br>
                      「この記事を見た人が思わず商品を購入したくなる魅力的な記事内容にしてください」などのプロンプトを埋め込むことができます。
                    </div>
                  </div>
                </div>
                <textarea name="custom_prompt" id="input-custom_prompt" rows="9" cols="80"><?php echo $form_items['custom_prompt']['val'] ?></textarea>
              </div>
              <?php if ($calliope_setting->gpt_version != CalliopeConfig::CALLIOPE_GPT3) : ?>
                <div class="input-wrapper">
                  <div class="input-header">
                    <div class="input-label"><?php echo $form_items['reference_urls']['label'] ?></div>
                    <div class="input-hint">
                      <div class="input-hint-label"><img src="<?php echo CalliopeConfig::get_img_uri(); ?>hint_icon.png"></div>
                      <div class="input-hint-text">
                        （注：「gpt-3.5-turbo」では使用できません。）記事生成時に参考にするページのURLを入力できます。改行で複数入力が可能です。</br>
                        特定の商品を販売する場合や、競合のページなどがある場合は、そのページの内容を参考に記事内容を生成できます。</br>
                        「カスタムプロンプト」と組み合わせることで、より柔軟な記事生成ができます。
                      </div>
                    </div>
                  </div>
                  <textarea name="reference_urls" id="input-reference_urls" rows="9" cols="80"><?php echo $form_items['reference_urls']['val'] ?></textarea>
                </div>
              <?php endif; ?>
              <div class="input-wrapper">
                <div class="input-header">
                  <div class="input-label"><?php echo $form_items['cron_interval']['label'] ?></div>
                  <div class="input-required">*</div>
                  <div class="input-hint">
                    <div class="input-hint-label"><img src="<?php echo CalliopeConfig::get_img_uri(); ?>hint_icon.png"></div>
                    <div class="input-hint-text">
                      自動投稿を生成する間隔を設定できます。「実行しない」を選択すると自動投稿は実行されません。</br>
                      もし間隔を設定しているのに自動実行が動かない場合は「メイン設定」の「自動投稿を実行する」が「実行する」になっていることを確認してください。（<a target="_blank" href="<?php echo admin_url('admin.php?page=calliope_menu') ?>">メイン設定</a>）
                    </div>
                  </div>
                </div>
                <select name="cron_interval" id="input-cron_interval" class="uk-select">
                  <?php foreach (CalliopeConfig::CALLIOPE_CRON_INTERVAL as $_idx => $label) : ?>
                    <option value="<?php echo $_idx ?>" <?php if ($_idx == $form_items['cron_interval']['val']) echo 'selected'; ?>><?php echo $label ?></option>
                  <?php endforeach; ?>
                </select>
              </div>
              <div class="input-index">投稿書式設定</div>
              <div class="input-wrapper">
                <div class="input-header">
                  <div class="input-label"><?php echo $form_items['is_view_lead_text']['label'] ?></div>
                  <div class="input-required">*</div>
                  <div class="input-hint">
                    <div class="input-hint-label"><img src="<?php echo CalliopeConfig::get_img_uri(); ?>hint_icon.png"></div>
                    <div class="input-hint-text">
                      リード文の表示・非表示を設定できます。</br>
                      リード文は投稿のはじめに表示される、200文字程度で記事全体を説明した文章です。
                    </div>
                  </div>
                </div>
                <select name="is_view_lead_text" id="input-is_view_lead_text" class="uk-select">
                  <?php foreach (CalliopeConfig::CALLIOPE_IS_VIEW_LEAD_TEXT as $_idx => $label) : ?>
                    <option value="<?php echo $_idx ?>" <?php if ($_idx == $form_items['is_view_lead_text']['val']) echo 'selected'; ?>><?php echo $label ?></option>
                  <?php endforeach; ?>
                </select>
              </div>
              <div class="input-wrapper">
                <div class="input-header">
                  <div class="input-label"><?php echo $form_items['index_count']['label'] ?></div>
                  <div class="input-required">*</div>
                  <div class="input-hint">
                    <div class="input-hint-label"><img src="<?php echo CalliopeConfig::get_img_uri(); ?>hint_icon.png"></div>
                    <div class="input-hint-text">
                      見出し件数を設定できます。</br>
                      ここで設定した件数分、見出し＋本文を生成します。
                    </div>
                  </div>
                </div>
                <select name="index_count" id="input-index_count" class="uk-select">
                  <?php foreach (CalliopeConfig::CALLIOPE_INDEX_COUNT_LABEL as $_idx => $label) : ?>
                    <option value="<?php echo $_idx ?>" <?php if ($_idx == $form_items['index_count']['val']) echo 'selected'; ?>><?php echo $label ?></option>
                  <?php endforeach; ?>
                </select>
              </div>
              <div class="input-wrapper" style="display:none" <?php //2024/02/16 非表示に変更
                                                              ?>>
                <div class="input-header">
                  <div class="input-label"><?php echo $form_items['is_view_index_ad']['label'] ?></div>
                  <div class="input-required">*</div>
                  <div class="input-hint">
                    <div class="input-hint-label"><img src="<?php echo CalliopeConfig::get_img_uri(); ?>hint_icon.png"></div>
                    <div class="input-hint-text">
                      見出しごとに広告を表示します。</br>
                      広告は本文の後ろに表示されます。
                    </div>
                  </div>
                </div>
                <select name="is_view_index_ad" id="input-is_view_index_ad" class="uk-select">
                  <?php foreach (CalliopeConfig::CALLIOPE_IS_VIEW_INDEX_AD as $_idx => $label) : ?>
                    <option value="<?php echo $_idx ?>" <?php if ($_idx == $form_items['is_view_index_ad']['val']) echo 'selected'; ?>><?php echo $label ?></option>
                  <?php endforeach; ?>
                </select>
              </div>
              <div class="input-wrapper">
                <div class="input-header">
                  <div class="input-label"><?php echo $form_items['is_view_summary']['label'] ?></div>
                  <div class="input-required">*</div>
                  <div class="input-hint">
                    <div class="input-hint-label"><img src="<?php echo CalliopeConfig::get_img_uri(); ?>hint_icon.png"></div>
                    <div class="input-hint-text">
                      要約の表示・非表示を設定できます。</br>
                      要約は記事の文末に表示される、全体を200文字程度でまとめた文章です。
                    </div>
                  </div>
                </div>
                <select name="is_view_summary" id="input-is_view_summary" class="uk-select">
                  <?php foreach (CalliopeConfig::CALLIOPE_IS_VIEW_SUMMARY as $_idx => $label) : ?>
                    <option value="<?php echo $_idx ?>" <?php if ($_idx == $form_items['is_view_summary']['val']) echo 'selected'; ?>><?php echo $label ?></option>
                  <?php endforeach; ?>
                </select>
              </div>
              <div class="input-wrapper" style="display:none" <?php //2024/02/16 非表示に変更
                                                              ?>>
                <div class="input-header">
                  <div class="input-label"><?php echo $form_items['is_view_last_ad']['label'] ?></div>
                  <div class="input-required">*</div>
                  <div class="input-hint">
                    <div class="input-hint-label"><img src="<?php echo CalliopeConfig::get_img_uri(); ?>hint_icon.png"></div>
                    <div class="input-hint-text">
                      記事文末の広告の表示・非表示を設定できます。</br>
                      広告は要約の後ろに表示されます。
                    </div>
                  </div>
                </div>
                <select name="is_view_last_ad" id="input-is_view_last_ad" class="uk-select">
                  <?php foreach (CalliopeConfig::CALLIOPE_IS_VIEW_LAST_AD as $_idx => $label) : ?>
                    <option value="<?php echo $_idx ?>" <?php if ($_idx == $form_items['is_view_last_ad']['val']) echo 'selected'; ?>><?php echo $label ?></option>
                  <?php endforeach; ?>
                </select>
              </div>
              <div class="input-wrapper">
                <div class="input-header">
                  <div class="input-label"><?php echo $form_items['embedded_html']['label'] ?></div>
                  <div class="input-hint">
                    <div class="input-hint-label"><img src="<?php echo CalliopeConfig::get_img_uri(); ?>hint_icon.png"></div>
                    <div class="input-hint-text">
                      HTMLタグを登録できます。登録したHTMLは記事内に埋め込まれます。</br>
                      広告タグなどを入力することも可能です。生成方法は各ASPの説明を確認してください。
                    </div>
                  </div>
                </div>
                <textarea name="embedded_html" id="input-embedded_html" rows="9" cols="80"><?php echo $form_items['embedded_html']['val'] ?></textarea>
              </div>
              <div class="input-index">投稿設定</div>
              <div class="input-wrapper">
                <div class="input-header">
                  <div class="input-label"><?php echo $form_items['status']['label'] ?></div>
                  <div class="input-required">*</div>
                  <div class="input-hint">
                    <div class="input-hint-label"><img src="<?php echo CalliopeConfig::get_img_uri(); ?>hint_icon.png"></div>
                    <div class="input-hint-text">
                      自動投稿する記事のステータスを設定できます。</br>
                      ステータスは「公開」「下書き」から選択できます。
                    </div>
                  </div>
                </div>
                <select name="status" id="input-status" class="uk-select">
                  <?php foreach (CalliopeConfig::CALLIOPE_STATUS as $_idx => $label) : ?>
                    <option value="<?php echo $_idx ?>" <?php if ($_idx == $form_items['status']['val']) echo 'selected'; ?>><?php echo $label ?></option>
                  <?php endforeach; ?>
                </select>
              </div>
              <div class="input-wrapper">
                <div class="input-header">
                  <div class="input-label"><?php echo $form_items['post_type']['label'] ?></div>
                  <div class="input-required">*</div>
                  <div class="input-hint">
                    <div class="input-hint-label"><img src="<?php echo CalliopeConfig::get_img_uri(); ?>hint_icon.png"></div>
                    <div class="input-hint-text">自動投稿される記事の投稿タイプを設定できます。</div>
                  </div>
                </div>
                <select name="post_type" id="input-post_type" class="uk-select">
                  <?php foreach (get_custom_post_types_labels_and_slugs() as $post_type) : ?>
                    <option value="<?php echo $post_type['slug'] ?>" <?php if ($post_type['slug'] == $form_items['post_type']['val']) echo 'selected'; ?>><?php echo $post_type['label'] ?></option>
                  <?php endforeach; ?>
                </select>
              </div>
              <div class="input-wrapper">
                <div class="input-header">
                  <div class="input-label"><?php echo $form_items['category']['label'] ?></div>
                  <div class="input-hint">
                    <div class="input-hint-label"><img src="<?php echo CalliopeConfig::get_img_uri(); ?>hint_icon.png"></div>
                    <div class="input-hint-text">
                      自動投稿される記事のカテゴリーを設定できます。</br>
                      カテゴリーがない場合は「<a target="_blank" href="<?php echo admin_url('edit-tags.php?taxonomy=category') ?>">カテゴリー</a>」から追加してください。
                    </div>
                  </div>
                </div>
                <?php $categories = get_categories(['hide_empty' => false]); ?>
                <?php if (!empty($categories)) : ?>
                  <?php foreach ($categories as $category) : ?>
                    <?php $checked = !empty($form_items['category']['val']) && in_array($category->term_id, $form_items['category']['val']) ? true : false ?>
                    <div>
                      <input id="input-category-<?php echo $category->term_id ?>" name="category[]" type="checkbox" <?php if ($checked) echo 'checked' ?> value="<?php echo $category->term_id ?>">
                      <label for="input-category-<?php echo $category->term_id ?>"><?php echo $category->name ?></label>
                    </div>
                  <?php endforeach; ?>
                <?php endif; ?>
              </div>
              <div class="input-wrapper">
                <div class="input-header">
                  <div class="input-label"><?php echo $form_items['tag']['label'] ?></div>
                  <div class="input-hint">
                    <div class="input-hint-label"><img src="<?php echo CalliopeConfig::get_img_uri(); ?>hint_icon.png"></div>
                    <div class="input-hint-text">
                      自動投稿される記事のタグを設定できます。</br>
                      タグがない場合は「<a target="_blank" href="<?php echo admin_url('edit-tags.php?taxonomy=post_tag') ?>">タグ</a>」から追加してください。
                    </div>
                  </div>
                </div>
                <?php $tags = get_tags(['hide_empty' => false]); ?>
                <?php if (!empty($tags)) : ?>
                  <?php foreach ($tags as $tag) : ?>
                    <?php $checked = !empty($form_items['tag']['val']) && in_array($tag->term_id, $form_items['tag']['val']) ? true : false ?>
                    <div>
                      <input id="input-tag-<?php echo $tag->term_id ?>" name="tag[]" type="checkbox" <?php if ($checked) echo 'checked' ?> value="<?php echo $tag->term_id ?>">
                      <label for="input-tag-<?php echo $tag->term_id ?>"><?php echo $tag->name ?></label>
                    </div>
                  <?php endforeach; ?>
                <?php endif; ?>
              </div>
              <div class="bottom-action-wrapper">
                <input type="hidden" name="submited" value="submited">
                <input type="hidden" name="id" value="<?php if ($id != null) echo $id; ?>">
                <button type="submit" name="submit-test-autogenerator" class="setting-test-autogenerator button button-green" value="test-autogenerator">保存してテスト実行する</button>
                <button type="submit" name="submit-save" class="setting-save button button-primary" value="save">変更を保存する</button>
              </div>
            </div>
            <div class="feed-post-format-wrapper">
              <div class="feed-post-format-label-wrapper">
                <div class=" feed-post-format-label">投稿書式</div>
                <div class="input-hint">
                  <div class="input-hint-label"><img src="<?php echo CalliopeConfig::get_img_uri(); ?>hint_icon.png"></div>
                  <div class="input-hint-text">実際に生成される投稿の書式です。左側の「投稿書式設定」を変更すると書式が反映されます。</div>
                </div>
              </div>
            </div>
          </div>
        </div>
      </div>
    </div>
    <?php $test_autogen_exec = $is_posted_submit_test_autogenerator ? 1 : 0; ?>
    <input type="hidden" id="test-autogen-exec" name="test-autogen-exec" value="<?php echo $test_autogen_exec ?>">
    <input type="hidden" id="calliope-feed-schedule-id" name="calliope-feed-schedule-id" value="<?php echo $cfs_id ?>">
  </form>
  <!-- モーダルエリアここから -->
  <section id="modalArea" class="modalArea">
    <div id="modalBg" class="modalBg"></div>
    <div class="modalWrapper">
      <div class="modalContents">
        <h2 id='test-autogen-title'>自動投稿のテストを実行しています</h2>
        <p>処理完了には5分ほどかかります。</p>
        <p>10分以上投稿が追加されない場合は再度テスト実行してみてください。</p>
        <p>テスト実行終了後、投稿へのリンクが表示されます。</p>
        <p id="status-str">ステータス ... 自動生成を開始します。</p>
        </p>
      </div>
    </div>
  </section>
  <!-- モーダルエリアここまで -->

  <?php calliope_wrap_tail(); ?>
<?php
}

function calliope_feed_edit_page_validation()
{
  $errors = [];
  $form_items = CalliopeConfig::CALLIOPE_FEED_EDIT_FORM_ITEMS;

  foreach ($form_items as $key => $form_item) {
    $form_items[$key] = get_post_value($key);
    if ($form_item['required'] == true && (get_post_value($key) == null || get_post_value($key) == '')) {
      $errors[] = [
        'label' => $form_item['label'],
        'message' => '必須項目です',
      ];
    }
  }

  return $errors;
}

function item_posted_value_init_feed_edit_page()
{
  $form_items = CalliopeConfig::CALLIOPE_FEED_EDIT_FORM_ITEMS;

  foreach ($form_items as $key => $form_item) {
    if ($key == 'category' || $key == 'tag') {
      $form_items[$key]['val'] = get_post_value($key) != null ? get_post_value($key) : [];
    } else {
      $form_items[$key]['val'] = get_post_value($key);
    }
  }

  return $form_items;
}

function get_custom_post_types_labels_and_slugs()
{
  $custom_post_types = get_post_types(
    array(
      'public' => true, // 公開されている投稿タイプのみを取得
      '_builtin' => false, // ビルトイン（デフォルトの）投稿タイプは除外
    ),
    'objects' // 'objects'を指定することで、詳細なオブジェクト情報を取得します
  );

  $custom_post_type_info = array();

  $custom_post_type_info[] = [
    'label' => '選択してください',
    'slug' => '',
  ];
  $custom_post_type_info[] = [
    'label' => '投稿',
    'slug' => 'post',
  ];

  foreach ($custom_post_types as $post_type) {
    // ラベルとスラッグを取得して配列に格納
    $labels_and_slug = [
      'label' => $post_type->labels->name,
      'slug' => $post_type->name,
    ];
    $custom_post_type_info[] = $labels_and_slug;
  }

  return $custom_post_type_info;
}


function calliope_feed_edit_page_test_autogen_ajax_handler()
{
  global $wpdb;

  $calliope_feeds_schedule_id = get_post_value('calliope_feeds_schedule_id');
  $calliope_feed_schedule = $wpdb->get_row("SELECT * FROM " . CalliopeConfig::TABLE_CALLIOPE_FEEDS_SCHEDULE . " WHERE id = " . $calliope_feeds_schedule_id);

  $message = '';
  switch ($calliope_feed_schedule->status) {
    case 'start':
      $message = 'テスト実行を開始しました';
      break;
    case 'related_keyword':
      $message = '関連キーワードを生成中';
      break;
    case 'regulation':
      $message = 'レギュレーションを生成中';
      break;
    case 'index':
      $message = '見出しを生成中';
      break;
    case 'contents':
      $message = 'コンテンツを生成中';
      break;
    case 'summary':
      $message = '要約を生成中';
      break;
    case 'lead_text':
      $message = 'リード文を生成中';
      break;
    case 'title':
      $message = 'タイトルを生成中';
      break;
    case 'done':
      $message = 'テスト実行が完了しました。';
      break;
    default:
      $message = 'コンテンツを生成中';
      break;
  }

  // 必要な処理を行い、レスポンスを返す
  $response = array(
    'code' => 'success',
    'status' => $calliope_feed_schedule->status,
    'message' => $message,
    'post_id' => $calliope_feed_schedule->post_id
  );

  wp_send_json($response);
}
add_action('wp_ajax_calliope_feed_edit_page_test_autogen_ajax_request', 'calliope_feed_edit_page_test_autogen_ajax_handler');
add_action('wp_ajax_nopriv_calliope_feed_edit_page_test_autogen_ajax_request', 'calliope_feed_edit_page_test_autogen_ajax_handler');


function calliope_feed_edit_page_related_keyword_generate_ajax_handler()
{
  CalliopeLogger::log('calliope_feed_edit_page_related_keyword_generate_ajax_handler ... ');

  $related_keywords = generate_related_keyword(get_post_value('keyword'));

  // 必要な処理を行い、レスポンスを返す
  $response = array(
    'code' => 'success',
    'related_keywords' => $related_keywords
  );

  wp_send_json($response);
}
add_action('wp_ajax_calliope_feed_edit_page_related_keyword_generate_ajax_request', 'calliope_feed_edit_page_related_keyword_generate_ajax_handler');
add_action('wp_ajax_nopriv_calliope_feed_edit_page_related_keyword_generate_ajax_request', 'calliope_feed_edit_page_related_keyword_generate_ajax_handler');

// 関連キーワードを自動生成する
function generate_related_keyword($keyword)
{
  $prompt_list = [
    new Prompt('「' . $keyword . '」というキーワードで悩みを解決したい人がGoogleで検索する際の関連キーワードの候補を20個挙げてください。'),
  ];

  $prompt_list = request_related_keyword($prompt_list);
  $response = $prompt_list[count($prompt_list) - 1]->response;

  $rks = get_lists_anser($response);

  foreach ($rks as $key => $related_keyword) {
    $related_keyword = str_replace($keyword . 'の', '', $related_keyword);
    $related_keyword = str_replace($keyword, '', $related_keyword);
    $related_keyword = str_replace('「', '', $related_keyword);
    $related_keyword = str_replace('」', '', $related_keyword);
    $related_keyword = str_replace(' ', '', $related_keyword);
    $related_keywords[] = $related_keyword;
  }

  return $related_keywords;
}

function request_related_keyword($prompt_list)
{
  $client = new Client;
  $messages_array = [];

  foreach ($prompt_list as $article_format) {
    $try_cnt = 0;

    while ($try_cnt < 10) {
      # messages_arrayにrole:user、リクエスト文を追加する。
      $messages_array[] = [
        "role" => "user",
        "content" => $article_format->prompt
      ];
      # messages_arrayでリクエストする。
      $res = $client->completions($messages_array);

      if (empty($res)) {
        // 最新のリクエストを削除する
        array_pop($messages_array);

        CalliopeLogger::log("リクエストに失敗しました。もう一度やります。");
      } else {
        $article_format->response = $res;

        # リクエストした内容をrole:systemにする。
        $messages_array[count($messages_array) - 1]["role"] = "system";
        # レスポンス内容をrole:assistantとしてmessages_arrayににする。
        $messages_array[] = [
          "role" => "assistant",
          "content" => $article_format->response
        ];

        break;
      }

      $try_cnt++;
    }
  }

  return $prompt_list;
}
