<?php

//=================================================
// サブメニュー ページ内容の表示・更新処理
//=================================================

require_once(dirname(__FILE__) . '/../util.php');
require_once(get_plugin_root() . '/config.php');
require_once(get_plugin_root() . '/db/operation.php');

function calliope_feeds_page()
{
  global $wpdb;

  //// レコードを削除する
  if (!empty(get_post_value('feed-remove'))) {
    CalliopeLogger::log('delete feed .... ' . get_post_value('feed-remove'));
    DBoperation::db_delete(CalliopeConfig::TABLE_CALLIOPE_FEEDS, ['id' => get_post_value('feed-remove')]);
  }

  // 一覧のデータを取得する
  $feeds = $wpdb->get_results('SELECT * FROM ' . CalliopeConfig::TABLE_CALLIOPE_FEEDS);

  ///
  //// 生成器字数制限を取得する
  $is_verify_post_count = false;
  // 記事生成制限を取得する
  $post_limit = get_post_limit();
  // 生成した有効記事数を取得する
  $monthly_active_post_count = get_monthly_active_post_count();

  if (!empty($post_limit)) {
    $is_verify_post_count = true;
  }

  ///// jsの読み込み
  wp_enqueue_script('calliope-scripts-calliope_feeds_page', get_plugin_root_uri('js/calliope_feeds_page.js'));
?>

  <?php calliope_wrap_head(); ?>

  <?php if (!empty(get_post_value('feed-remove'))) : ?>
    <div class="updated">
      <p><strong>フィードを削除しました</strong></p>
    </div>
  <?php endif; ?>

  <form method="post" name="form" action="<?php echo str_replace('%7E', '~', $_SERVER['REQUEST_URI']); ?>">
    <div class="calliope_feeds_page">
      <div class="calliope-area">
        <div class="calliope-wrapper">
          <div class="calliope-header">
            <h1>フィード設定</h1>
            <div class="feed-header">
              <?php if ($is_verify_post_count) : ?>
                <div class="feed-count"><strong>今月の生成記事数 <?php echo $monthly_active_post_count ?>件 / <?php echo $post_limit ?>件中</strong></div>
              <?php endif; ?>
              <a href="<?php echo admin_url('admin.php?page=calliope_feed_edit&action=edit') ?>">
                <button type="button" name="submit-new-feed" class="setting-new-feed button button-green" value="new-feed">新規登録</button>
              </a>
            </div>
          </div>
          <div class="calliope-contents-area">
            <div class="calliope-contents-wrapper">
              <?php if ($feeds) : ?>
                <?php foreach ($feeds as $feed) : ?>
                  <div class="feed-list-row margin-top-40">
                    <div class="feed-list-row-left-area">
                      <div class="feed-list-title"><a href="<?php echo admin_url('admin.php?page=calliope_feed_edit&action=edit&id=' . $feed->id) ?>"><?php echo $feed->name ?></a></div>
                      <div class="feed-list-prop-wrapper margin-top-20">
                        <div class="feed-list-prop">
                          <div class="feed-list-prop-head">キーワード : </div><div><?php echo $feed->keyword ?></div>
                        </div>
                        <div class="feed-list-prop">
                          <div class="feed-list-prop-head">スケジュール : </div><div><?php echo CalliopeConfig::CALLIOPE_CRON_INTERVAL[$feed->cron_interval] ?></div>
                        </div>
                        <div class="feed-list-prop">
                          <div class="feed-list-prop-head">投稿ステータス : </div><div><?php echo CalliopeConfig::CALLIOPE_STATUS[$feed->status] ?></div>
                        </div>
                        <div class="feed-list-prop">
                          <div class="feed-list-prop-head">最終実行時間 : </div><div><?php echo $feed->last_cron_executed ?></div>
                        </div>
                      </div>
                    </div>
                    <div class="feed-list-row-right-area">
                      <div class="feed-remove" value="<?php echo $feed->id ?>">
                        <div class="feed-remove-icon"><img src="<?php echo CalliopeConfig::get_img_uri(); ?>trash.png"></div>
                      </div>
                    </div>
                  </div>
                  <div class="margin-top-20"><span class="devider"></span></div>
                <?php endforeach; ?>
              <?php else : ?>
                <div><strong>レコードがありません。「新規登録」からフィードを登録してください。</strong></div>
              <?php endif; ?>
            </div>
          </div>
        </div>
      </div>
    </div>
  </form>

  <?php calliope_wrap_tail(); ?>

<?php
}
