<?php

//=================================================
// サブメニュー キーワード生成画面
//=================================================

require_once(dirname(__FILE__) . '/../util.php');
require_once(get_plugin_root() . '/config.php');
require_once(get_plugin_root() . '/calliope_logger.php');
require_once(get_plugin_root() . '/autogenerator/prompt.php');
require_once(get_plugin_root() . '/autogenerator/openai/client.php');

function calliope_keyword_page()
{
  $form_items = CalliopeConfig::CALLIOPE_KEYWORD_FORM_ITEMS;

  $errors = [];
  $related_keywords = [];

  if (get_post_value('posted') == 'save') {

    $errors = calliope_keyword_page_validation();

    if (empty($errors)) {
      $related_keywords = generate_related_keyword(get_post_value('keyword'));
      $form_items['keyword']['val'] = get_post_value('keyword');
    }
  }

  ///// jsの読み込み
  wp_enqueue_script('calliope-scripts', get_plugin_root_uri('js/calliope_script.js'));

?>

  <?php calliope_wrap_head(); ?>

  <?php foreach ($errors as $error) : ?>
    <div class="error">
      <p><strong><?php echo $error['label'] ?>は<?php echo $error['message'] ?></strong></p>
    </div>
  <?php endforeach; ?>

  <form method="post" action="<?php echo str_replace('%7E', '~', $_SERVER['REQUEST_URI']); ?>">
    <div class="calliope_keyword_page">
      <div class="calliope-area">
        <div class="calliope-wrapper">
          <div class="calliope-header">
            <h1>キーワード生成</h1>
          </div>
          <div class="calliope-contents-area">
            <div class="calliope-contents-wrapper">
              <div class="input-wrapper">
                <div class="input-header">
                  <div class="input-label">キーワード</div>
                  <div class="input-required">*</div>
                  <div class="input-hint">
                    <div class="input-hint-label"><img src="<?php echo CalliopeConfig::get_img_uri(); ?>hint_icon.png"></div>
                    <div class="input-hint-text">キーワードを入力してください。画面下の「キーワード生成」を押すと関連したキーワードを20個生成します。</div>
                  </div>
                </div>
                <input type="text" name="keyword" value="<?php echo $form_items['keyword']['val'] ?>">
              </div>
              <?php if (!empty($related_keywords)) : ?>
                <div class="related-keyword-label">生成したキーワードはコピーした後、フィード編集画面の「関連キーワード」のテキストエリアに貼り付けてください。</div>
                <div class="related-keyword-gen-area">
                  <?php foreach ($related_keywords as $related_keyword) : ?>
                    <div class="gen-related-keyword"><?php echo $related_keyword; ?></div>
                  <?php endforeach; ?>
                </div>
              <?php endif; ?>
              <div class="bottom-action-wrapper">
                <input type="hidden" name="posted" value="save">
                <button type="submit" name="submit" class="submit-btn button button-primary" value="save">キーワード生成</button>
              </div>
            </div>
          </div>
        </div>
      </div>
    </div>
  </form>

  <?php calliope_wrap_tail(); ?>
<?php
}

function calliope_keyword_page_validation()
{
  $errors = [];
  $form_items = CalliopeConfig::CALLIOPE_KEYWORD_FORM_ITEMS;

  foreach ($form_items as $key => $form_item) {
    $form_items[$key] = get_post_value($key);
    if ($form_item['required'] == true && (get_post_value($key) == null || get_post_value($key) == '')) {
      $errors[] = [
        'label' => $form_item['label'],
        'message' => '必須項目です',
      ];
    }
  }

  return $errors;
}