<?php

//=================================================
// サブメニュー ページ内容の表示・更新処理
//=================================================

require_once(dirname(__FILE__) . '/../util.php');
require_once(get_plugin_root() . '/config.php');
require_once(get_plugin_root() . '/db/operation.php');

function calliope_setting_page()
{
  global $wpdb;

  $form_items = CalliopeConfig::CALLIOPE_SETTING_FORM_ITEMS;
  $errors = [];

  if (get_post_value('posted') == 'save') {
    //設定画面で入力された設定値を保存

    // 選択したGPTのバージョンが正常に動作するかを確認する
    $errors = check_valid_gpt_version();

    if (empty($errors)) {
      $cols = [];
      foreach ($form_items as $key => $form_item) {
        $cols[$key] = get_post_value($key);
      }

      $result = DBoperation::db_update(CalliopeConfig::TABLE_CALLIOPE_SETTING, $cols, ['id' => 1]);

      if ($result === FALSE || $result < 1) {
        DBoperation::db_insert(CalliopeConfig::TABLE_CALLIOPE_SETTING, $cols);
      }
    }
  }

  $calliope_setting = $wpdb->get_row("SELECT * FROM " . CalliopeConfig::TABLE_CALLIOPE_SETTING . " WHERE id = 1");
  if ($calliope_setting != null) {
    foreach ($form_items as $key => $form_item) {
      $form_items[$key]['val'] = $calliope_setting->{$key};
    }
  }

  // バリデーションエラー時の初期値を設定する
  if (!empty($errors)) {
    $form_items = item_posted_value_init_setting_page();
  }

  ///// フラグを整理
  $is_posted_submit_save = get_post_value('posted') == 'save' && empty($errors) ? true : false;

  ///// jsの読み込み
  wp_enqueue_script('calliope-scripts', get_plugin_root_uri('js/calliope_script.js'));

?>

  <?php calliope_wrap_head(); ?>

  <?php if ($is_posted_submit_save) : ?>
    <div class="updated">
      <p><strong>設定を保存しました</strong></p>
    </div>
  <?php endif; ?>

  <?php foreach ($errors as $error) : ?>
    <div class="error"><?php echo $error['message'] ?></div>
  <?php endforeach; ?>

  <form method="post" action="<?php echo str_replace('%7E', '~', $_SERVER['REQUEST_URI']); ?>">
    <div class="calliope_setting_page">
      <div class="calliope-area">
        <div class="calliope-wrapper">
          <div class="calliope-header">
            <h1>メイン設定</h1>
          </div>
          <div class="calliope-contents-area">
            <div class="calliope-contents-wrapper">
              <div class="input-index">基本設定</div>
              <div class="input-wrapper">
                <div class="input-header">
                  <div class="input-label"><?php echo $form_items['open_ai_api_key']['label'] ?></div>
                  <div class="input-hint">
                    <div class="input-hint-label"><img src="<?php echo CalliopeConfig::get_img_uri(); ?>hint_icon.png"></div>
                    <div class="input-hint-text">OpenAIのAPIキーを入力してください。OpenAIのAPIキー発行手順は<a target="_blank" href="<?php echo admin_url('admin.php?page=calliope_manual') ?>">マニュアル</a>を参照してください。</div>
                  </div>
                </div>
                <input type="text" name="open_ai_api_key" value="<?php echo $form_items['open_ai_api_key']['val'] ?>">
              </div>
              <div class="input-wrapper">
                <div class="input-header">
                  <div class="input-label"><?php echo $form_items['is_cron_exec']['label'] ?></div>
                  <div class="input-hint">
                    <div class="input-hint-label"><img src="<?php echo CalliopeConfig::get_img_uri(); ?>hint_icon.png"></div>
                    <div class="input-hint-text">投稿を自動で生成するかどうかの設定です。「実行する」を選択した場合、フィード設定の「実行スケジュール」で設定した時間ごとに投稿を自動的に生成します。</div>
                  </div>
                </div>
                <select name="is_cron_exec" id="input-is_cron_exec" class="uk-select">
                  <?php foreach (CalliopeConfig::CALLIOPE_IS_CRON_EXEC as $_idx => $label) : ?>
                    <option value="<?php echo $_idx ?>" <?php if ($_idx == $form_items['is_cron_exec']['val']) echo 'selected'; ?>><?php echo $label ?></option>
                  <?php endforeach; ?>
                </select>
              </div>
              <div class="input-wrapper">
                <div class="input-header">
                  <div class="input-label"><?php echo $form_items['gpt_version']['label'] ?></div>
                  <div class="input-hint">
                    <div class="input-hint-label"><img src="<?php echo CalliopeConfig::get_img_uri(); ?>hint_icon.png"></div>
                    <div class="input-hint-text">記事生成に利用するGPTのバージョンを設定します。生成する記事のクオリティを保つため、基本的には「gpt-4-turbo」を選択してください。GPT-4の利用にはOpenAI有料版の登録が必要です。</div>
                  </div>
                </div>
                <select name="gpt_version" id="input-gpt_version" class="uk-select">
                  <?php foreach (CalliopeConfig::CALLIOPE_GPT_VERSION as $_idx => $label) : ?>
                    <option value="<?php echo $_idx ?>" <?php if ($_idx == $form_items['gpt_version']['val']) echo 'selected'; ?>><?php echo $label ?></option>
                  <?php endforeach; ?>
                </select>
              </div>
              <div class="bottom-action-wrapper">
                <input type="hidden" name="posted" value="save">
                <button type="submit" name="submit" class="submit-btn button button-primary" value="save">設定を保存</button>
              </div>
            </div>
          </div>
        </div>
      </div>
    </div>
  </form>

  <?php calliope_wrap_tail(); ?>

<?php
}

function item_posted_value_init_setting_page()
{
  $form_items = CalliopeConfig::CALLIOPE_SETTING_FORM_ITEMS;

  foreach ($form_items as $key => $form_item) {
    $form_items[$key]['val'] = get_post_value($key);
  }

  return $form_items;
}

// 関連キーワードを自動生成する
function check_valid_gpt_version()
{
  $prompt_list = [
    new Prompt('テストプロンプト'),
  ];

  $errors = [];

  $status = request_check_valid_gpt_version($prompt_list);
  if (!$status) {
    $errors[] = [
      'label' => 'GPTのバージョン',
      'message' => 'このAPIキーは選択したGPTのバージョンに対応していません。OpenAIの有料版を利用しているか確認してください。',
    ];
  }

  return $errors;
}

function request_check_valid_gpt_version($prompt_list)
{
  $client = new Client(get_post_value('open_ai_api_key'), get_post_value('gpt_version'));

  $status = false;

  foreach ($prompt_list as $article_format) {
    # messages_arrayにrole:user、リクエスト文を追加する。
    $messages_array[] = [
      "role" => "user",
      "content" => $article_format->prompt
    ];
    # messages_arrayでリクエストする。
    $res = $client->completions($messages_array);

    if (!empty($res)) {
      $status = true;
    } else {
      CalliopeLogger::log("リクエストに失敗しました。GPTのバージョンに対応していません");
    }
  }

  return $status;
}
