<?php

//=================================================
// サブメニュー サブスクリプション登録画面
//=================================================

require_once(dirname(__FILE__) . '/../util.php');
require_once(get_plugin_root() . '/config.php');
require_once(get_plugin_root() . '/api/stripe.php');
require_once(get_plugin_root() . '/db/operation.php');

function calliope_subsc_purchase_page()
{
  global $wpdb;

  // ユーザーのメールアドレス
  $calliope_user = $wpdb->get_row("SELECT * FROM " . CalliopeConfig::TABLE_CALLIOPE_USER . " WHERE id = 1");
  $email = $calliope_user->mail;

  $stripe = new Stripe();

  // プランの変更
  if (get_post_value('plan_update') == 'update') {
    $stripe->plan_update($email, get_post_value('price_id'));
  }

  // プランの解約
  if (get_post_value('plan_cancel') == 'cancel') {
    $stripe->plan_cancel($email);
  }

  // 商品情報を取得する
  $p_result = $stripe->get_products();
  $products = $p_result['data']->products;
  usort($products, "sort_product");

  // サブスクリプションの登録状況を取得する
  $active_subsc_product_id = '';
  $s_result = $stripe->check_subscription_status($email);

  if ($s_result['code'] == ApiBase::SUCCESS) {
    if ($s_result['data']->status == "active") {
      $active_subsc_product_id = $s_result['data']->product_id;
    }
  }

  ///// jsの読み込み
  wp_enqueue_script('calliope-scripts', get_plugin_root_uri('js/calliope_script.js'));
  wp_enqueue_script('calliope-scripts-calliope_subsc_purchase_page', get_plugin_root_uri('js/calliope_subsc_purchase_page.js'));

?>

  <?php calliope_wrap_head(); ?>

  <?php if (get_get_value('result') == 'success') : ?>
    <div class="updated">
      <p><strong>サブスクリプションの登録が完了しました。</strong></p>
      <p><strong>登録内容の反映までに時間がかかる場合があります。時間をおいてご確認ください</strong></p>
    </div>
  <?php endif; ?>

  <?php if (get_post_value('plan_update') == 'update') : ?>
    <div class="updated">
      <p><strong>サブスクリプションのプランを変更しました。</strong></p>
      <p><strong>登録内容の反映までに時間がかかる場合があります。時間をおいてご確認ください</strong></p>
    </div>
  <?php endif; ?>

  <?php if (get_post_value('plan_cancel') == 'cancel') : ?>
    <div class="updated">
      <p><strong>サブスクリプションの解約が完了しました。</strong></p>
      <p><strong>登録内容の反映までに時間がかかる場合があります。時間をおいてご確認ください</strong></p>
    </div>
  <?php endif; ?>

  <div class="calliope_subsc_purchase_page">
    <div class="calliope-area">
      <div class="calliope-wrapper">
        <div class="calliope-header">
          <h1>サブスクリプション登録</h1>
        </div>
        <div class="calliope-contents-area">
          <div class="plan-lists-wrapper">
            <div class="plan-lists">
              <?php foreach ($products as $product) : ?>
                <?php if (is_test_mode() == false && $product->metadata->test_mode == "true") continue; ?>
                <div class="plan-list">
                  <div class="plan-name"><?php echo $product->name; ?></div>
                  <div class="plan-description"><?php echo $product->metadata->description; ?></div>
                  <div class="plan-price">
                    <div class="price-label"><?php echo number_format($product->price); ?></div>
                    <div class="price-en-mon">円/月</div>
                  </div>
                  <div class="plan-post-limit">
                    <div class="check">✓</div>
                    <div class="plan-post-limit-label">
                      記事の作成 :
                      <?php if ($product->metadata->post_limit == 'no-limit') : ?>
                        無制限
                      <?php else : ?>
                        <?php echo $product->metadata->post_limit; ?>件
                      <?php endif; ?>
                    </div>
                  </div>
                  <?php /*
                  <div class="plan-site-limit">
                    <div class="check">✓</div>
                    <div class="plan-site-limit-label">
                      複数サイト運用 :
                      <?php if ($product->metadata->site_limit == 'no-limit') : ?>
                        無制限
                      <?php else : ?>
                        <?php echo $product->metadata->site_limit; ?>件
                      <?php endif; ?>
                    </div>
                  </div>
                  */ ?>
                  <div class="check-list">
                    <div class="check">✓</div>
                    <div class="check-list-label">フィード登録</div>
                  </div>
                  <div class="check-list">
                    <div class="check">✓</div>
                    <div class="check-list-label">記事の自動生成</div>
                  </div>
                  <?php if (empty($active_subsc_product_id)) : ?>
                    <form action="https://stripehandler-3arzn46uba-uc.a.run.app" method="POST">
                      <div class="checkout-btn">
                        <input type="hidden" name="email" value="<?php echo $email ?>">
                        <input type="hidden" name="price_id" value="<?php echo $product->price_id ?>">
                        <button type="submit" role="link">
                          登録する
                        </button>
                      </div>
                    </form>
                  <?php elseif ($active_subsc_product_id == $product->product_id) : ?>
                    <div class="subscripted-label">登録済みです</div>
                    <form action="<?php echo str_replace('%7E', '~', $_SERVER['REQUEST_URI']); ?>" method="POST" onSubmit="return checkCancelApprove()">
                      <div class="cancel-btn">
                        <input type="hidden" name="price_id" value="<?php echo $product->price_id ?>">
                        <button type="submit" role="link" name="plan_cancel" value="cancel">
                          解約する
                        </button>
                      </div>
                    </form>
                  <?php else : ?>
                    <form action="<?php echo str_replace('%7E', '~', $_SERVER['REQUEST_URI']); ?>" method="POST" onSubmit="return checkPlanUpdateApprove()">
                      <div class="update-btn">
                        <input type="hidden" name="price_id" value="<?php echo $product->price_id ?>">
                        <button type="submit" role="link" name="plan_update" value="update">
                          プラン変更
                        </button>
                      </div>
                    </form>
                  <?php endif; ?>
                </div>
              <?php endforeach; ?>
            </div>
          </div>
        </div>
      </div>
    </div>
  </div>
  <script type="text/javascript">
  </script>

  <?php calliope_wrap_tail(); ?>
<?php
}

function sort_product($a, $b)
{
  $cmp = strcmp($a->metadata->index, $b->metadata->index);

  return $cmp;
}
