<?php

//=================================================
// サブメニュー ユーザー情報の登録ページ
//=================================================

require_once(dirname(__FILE__) . '/../util.php');
require_once(get_plugin_root() . '/config.php');
require_once(get_plugin_root() . '/api/user_certification.php');
require_once(get_plugin_root() . '/db/operation.php');

function calliope_user_create_page()
{
  $form_items = CalliopeConfig::CALLIOPE_USER_CREATE_FORM_ITEMS;

  $result = [];
  $errors = [];

  if (get_post_value('posted') == 'save') {

    $errors = calliope_user_create_page_validation();

    if (empty($errors)) {
      $uc = new UserCertification();
      $result = $uc->user_create(get_post_value('mail'), get_post_value('password'));

      if ($result['code'] == ApiBase::SUCCESS) {
        // ユーザーを作成
        $cols = [
          'mail' => get_post_value('mail'),
          'uid' => $result['data']->uid,
          'is_user_created' => 1,
        ];

        $_result = DBoperation::db_update(CalliopeConfig::TABLE_CALLIOPE_USER, $cols, ['id' => 1]);

        if ($_result === FALSE || $_result < 1) {
          DBoperation::db_insert(CalliopeConfig::TABLE_CALLIOPE_USER, $cols);
        }

        // リダイレクト処理
        if (check_is_coupon_code_registration()) {
          // クーポンコードを入力したことがある場合は、サブスクの登録画面に移動する
          redirect_subsuc_purchase_page();
        } else {
          // それ以外はサブスク登録orクーポンコード入力画面に移動する
          redirect_subsuc_or_coupon_page();
        }
      }
    }
  }

  ///// jsの読み込み
  wp_enqueue_script('calliope-scripts', get_plugin_root_uri('js/calliope_script.js'));

?>
  <?php calliope_wrap_head(); ?>

  <?php foreach ($errors as $error) : ?>
    <div class="error">
      <p><strong><?php echo $error['label'] ?>は <?php echo $error['message'] ?></strong></p>
    </div>
  <?php endforeach; ?>

  <?php if (!empty($result) && $result['code'] == ApiBase::ERROR) : ?>
    <div class="error">
      <p><strong><?php echo $result['message'] ?></strong></p>
    </div>
  <?php endif; ?>

  <form method="post" action="<?php echo str_replace('%7E', '~', $_SERVER['REQUEST_URI']); ?>">
    <div class="calliope_setting_page">
      <div class="calliope-area">
        <div class="calliope-wrapper">
          <div class="calliope-header">
            <h1>ユーザー情報登録</h1>
          </div>
          <div class="calliope-contents-area">
            <div class="calliope-contents-wrapper">
              <div class="input-wrapper">
                <div class="input-header">
                  <div class="input-label"><?php echo $form_items['mail']['label'] ?></div>
                  <div class="input-hint">
                    <div class="input-hint-label"><img src="<?php echo CalliopeConfig::get_img_uri(); ?>hint_icon.png"></div>
                    <div class="input-hint-text">
                      メールアドレスを入力してください。
                    </div>
                  </div>
                </div>
                <input type="text" name="mail" value="<?php echo $form_items['mail']['val'] ?>">
              </div>
              <div class="input-wrapper">
                <div class="input-header">
                  <div class="input-label"><?php echo $form_items['password']['label'] ?></div>
                  <div class="input-hint">
                    <div class="input-hint-label"><img src="<?php echo CalliopeConfig::get_img_uri(); ?>hint_icon.png"></div>
                    <div class="input-hint-text">
                      パスワードを入力してください。
                    </div>
                  </div>
                </div>
                <input type="text" name="password" value="<?php echo $form_items['password']['val'] ?>">
              </div>
              <div class="input-wrapper">
                <div class="input-header">
                  <strong>すでに登録済みの方は<a href="<?php echo admin_url('admin.php?page=calliope_user_login') ?>">ログイン画面</a>からログインしてください。</strong>
                </div>
              </div>
              <div class="bottom-action-wrapper">
                <input type="hidden" name="posted" value="save">
                <button type="submit" name="submit" class="submit-btn button button-primary" value="save">ユーザー情報を登録する</button>
              </div>
            </div>
          </div>
        </div>
      </div>
    </div>
  </form>

  <?php calliope_wrap_tail(); ?>
<?php
}


function calliope_user_create_page_validation()
{
  $errors = [];
  $form_items = CalliopeConfig::CALLIOPE_USER_CREATE_FORM_ITEMS;

  foreach ($form_items as $key => $form_item) {
    $form_items[$key] = get_post_value($key);
    if ($form_item['required'] == true && (get_post_value($key) == null || get_post_value($key) == '')) {
      $errors[] = [
        'label' => $form_item['label'],
        'message' => '必須項目です',
      ];
    }
  }

  return $errors;
}
