<?php

//=================================================
// サブメニュー ユーザー情報の登録ページ
//=================================================

require_once(dirname(__FILE__) . '/../util.php');
require_once(get_plugin_root() . '/config.php');
require_once(get_plugin_root() . '/api/user_certification.php');
require_once(get_plugin_root() . '/db/operation.php');

function calliope_user_login_page()
{
  $form_items = CalliopeConfig::CALLIOPE_USER_LOGIN_FORM_ITEMS;

  $result = [];
  $errors = [];

  $uc = new UserCertification();

  if (get_post_value('posted') == 'save') {

    $errors = calliope_user_login_page_validation();

    if (empty($errors)) {
      // まずidTokenを取得する
      $result = $uc->get_user_id_token_with_email_password(get_post_value('mail'), get_post_value('password'));

      if ($result['code'] == ApiBase::SUCCESS) {
        // idTokenの取得完了後にログイン処理を行う
        $result = $uc->user_login($result['data']->idToken);

        if ($result['code'] == ApiBase::SUCCESS) {
          // ログイン処理を行う

          // ユーザーを作成 or updateする
          $cols = [
            'mail' => get_post_value('mail'),
            'uid' => $result['data']->uid,
            'is_user_created' => 1,
          ];

          $_result = DBoperation::db_update(CalliopeConfig::TABLE_CALLIOPE_USER, $cols, ['id' => 1]);

          if ($_result === FALSE || $_result < 1) {
            DBoperation::db_insert(CalliopeConfig::TABLE_CALLIOPE_USER, $cols);
          }

          // リダイレクト処理
          if (is_verified()) {
            // サブスク登録 or クーポンコード入力済みの場合はメニューに飛ばす
            redirect_menu_page();
          } else {
            if (check_is_coupon_code_registration()) {
              // クーポンの入力が終わっている場合はサブスク登録画面に飛ばす
              redirect_subsuc_purchase_page();
            } else {
              // それ以外はサブスク登録orクーポンコード入力画面に移動する
              redirect_subsuc_or_coupon_page();
            }
          }
        }
      }
    }
  }

  ///// jsの読み込み
  wp_enqueue_script('calliope-scripts', get_plugin_root_uri('js/calliope_script.js'));
  wp_enqueue_script('calliope-scripts-calliope_user_login_page', get_plugin_root_uri('js/calliope_user_login_page.js'));
?>

  <?php calliope_wrap_head(); ?>

  <?php foreach ($errors as $error) : ?>
    <div class="error">
      <p><strong><?php echo $error['label'] ?>は <?php echo $error['message'] ?></strong></p>
    </div>
  <?php endforeach; ?>

  <?php if (!empty($result) && $result['code'] == ApiBase::ERROR) : ?>
    <div class="error">
      <p><strong><?php echo $result['message'] ?></strong></p>
    </div>
  <?php endif; ?>

  <form method="post" name="form" action="<?php echo str_replace('%7E', '~', $_SERVER['REQUEST_URI']); ?>">
    <div class="calliope_setting_page">
      <div class="calliope-area">
        <div class="calliope-wrapper">
          <div class="calliope-header">
            <h1>ログイン</h1>
          </div>
          <div class="calliope-contents-area">
            <div class="calliope-contents-wrapper">
              <div class="input-wrapper">
                <div class="input-header">
                  <div class="input-label"><?php echo $form_items['mail']['label'] ?></div>
                  <div class="input-hint">
                    <div class="input-hint-label"><img src="<?php echo CalliopeConfig::get_img_uri(); ?>hint_icon.png"></div>
                    <div class="input-hint-text">
                      メールアドレスを入力してください。
                    </div>
                  </div>
                </div>
                <input type="text" name="mail" value="<?php echo $form_items['mail']['val'] ?>">
              </div>
              <div class="input-wrapper">
                <div class="input-header">
                  <div class="input-label"><?php echo $form_items['password']['label'] ?></div>
                  <div class="input-hint">
                    <div class="input-hint-label"><img src="<?php echo CalliopeConfig::get_img_uri(); ?>hint_icon.png"></div>
                    <div class="input-hint-text">
                      パスワードを入力してください。
                    </div>
                  </div>
                </div>
                <input type="text" name="password" value="<?php echo $form_items['password']['val'] ?>">
              </div>
              <div class="bottom-action-wrapper">
                <input type="hidden" name="posted" value="save">
                <a href="<?php echo admin_url('admin.php?page=calliope_resest_password') ?>">パスワードをリセットする</a>
                <button type="submit" name="login" id="login-btn" class="submit-btn button button-primary" value="save" style="margin-left:15px;">ログイン</button>
              </div>
            </div>
          </div>
        </div>
      </div>
    </div>
  </form>

  <?php calliope_wrap_tail(); ?>
<?php
}


function calliope_user_login_page_validation()
{
  $errors = [];
  $form_items = CalliopeConfig::CALLIOPE_USER_LOGIN_FORM_ITEMS;

  foreach ($form_items as $key => $form_item) {
    $form_items[$key] = get_post_value($key);
    if ($form_item['required'] == true && (get_post_value($key) == null || get_post_value($key) == '')) {
      $errors[] = [
        'label' => $form_item['label'],
        'message' => '必須項目です',
      ];
    }
  }

  return $errors;
}
