<?php

require_once(dirname(__FILE__) . '/../util.php');
require_once(get_plugin_root() . '/calliope_logger.php');
require_once(get_plugin_root() . '/config.php');
require_once(get_plugin_root() . '/api/base.php');

class Mode extends ApiBase
{
  const IS_TEST = 'is_test';
  const IS_NOT_TEST = 'is_not_test';

  function get_mode($uid)
  {
    // POSTする
    $data = [
      'uid' => $uid
    ];
    $response = request_post(get_endpoint('getMode'), $data);

    // エラー発生時
    if (is_wp_error($response)) {
      $_m = 'APIリクエストが失敗しました(get_mode): ' . $response->get_error_message();
      CalliopeLogger::log($_m);

      return return_response(self::ERROR, '', $_m);
    }

    // レスポンスコード
    $response_code = wp_remote_retrieve_response_code($response);

    // メッセージ
    $body = get_body($response);
    $message_code = get_object_value($body, 'message_code');
    $message = get_object_value($body, 'message');

    // レスポンスのデータを処理する
    if ($response_code === 200) {
      return return_response(self::SUCCESS, $message_code, 'ユーザーのモードを取得しました。', $body);
    } else {
      $_m = 'APIリクエストが失敗しました(get_mode): ' . $message . '(' . $response["response"]["message"] . ')';
      CalliopeLogger::log($_m);

      return return_response(self::ERROR, $message_code, $message);
    }
  }

  function set_mode($uid, $mode, $passcode)
  {
    // POSTする
    $data = [
      'uid' => $uid,
      'mode' => $mode,
      'passcode' => $passcode,
    ];
    $response = request_post(get_endpoint('setMode'), $data);

    // エラー発生時
    if (is_wp_error($response)) {
      $_m = 'APIリクエストが失敗しました(set_mode): ' . $response->get_error_message();
      CalliopeLogger::log($_m);

      return return_response(self::ERROR, '', $_m);
    }

    // レスポンスコード
    $response_code = wp_remote_retrieve_response_code($response);

    // メッセージ
    $body = get_body($response);
    $message_code = get_object_value($body, 'message_code');
    $message = get_object_value($body, 'message');

    // レスポンスのデータを処理する
    if ($response_code === 200) {
      return return_response(self::SUCCESS, $message_code, 'ユーザーのモードを変更しました。', $body);
    } else {
      $_m = 'APIリクエストが失敗しました(set_mode): ' . $message . '(' . $response["response"]["message"] . ')';
      CalliopeLogger::log($_m);

      return return_response(self::ERROR, $message_code, $message);
    }
  }

}