<?php

require_once(dirname(__FILE__) . '/../util.php');
require_once(get_plugin_root() . '/calliope_logger.php');
require_once(get_plugin_root() . '/config.php');
require_once(get_plugin_root() . '/api/base.php');

class Stripe extends ApiBase
{
  function get_products()
  {
    // POSTする
    $data = [];
    $response = request_post($api_url = get_endpoint('getProducts'), $data);

    // エラー発生時
    if (is_wp_error($response)) {
      $_m = 'APIリクエストが失敗しました(get_products()): ' . $response->get_error_message();
      CalliopeLogger::log($_m);

      return return_response(self::ERROR, '', $_m);
    }

    // レスポンスコード
    $response_code = wp_remote_retrieve_response_code($response);

    // メッセージ
    $body = get_body($response);
    $message_code = get_object_value($body, 'message_code');
    $message = get_object_value($body, 'message');

    // レスポンスのデータを処理する
    if ($response_code === 200) {
      return return_response(self::SUCCESS, $message_code, '商品情報を取得しました', $body);
    } else {
      $_m = 'APIリクエストが失敗しました(get_products): ' . $message . '(' . $response["response"]["message"] . ')';
      CalliopeLogger::log($_m);

      return return_response(self::ERROR, $message_code, $message);
    }
  }

  function check_subscription_status($email)
  {
    // POSTする
    $data = [
      'email' => $email
    ];
    $response = request_post(get_endpoint('checkSubscriptionStatus'), $data);

    // エラー発生時
    if (is_wp_error($response)) {
      $_m = 'APIリクエストが失敗しました(check_subscription_status): ' . $response->get_error_message();
      CalliopeLogger::log($_m);

      return return_response(self::ERROR, '', $_m);
    }

    // レスポンスコード
    $response_code = wp_remote_retrieve_response_code($response);

    // メッセージ
    $body = get_body($response);
    $message_code = get_object_value($body, 'message_code');
    $message = get_object_value($body, 'message');

    // レスポンスのデータを処理する
    if ($response_code === 200) {
      return return_response(self::SUCCESS, $message_code, 'サブスクリプション登録状況を取得しました', $body);
    } else {
      $_m = 'APIリクエストが失敗しました(check_subscription_status): ' . $message . '(' . $response["response"]["message"] . ')';
      CalliopeLogger::log($_m);

      return return_response(self::ERROR, $message_code, $message);
    }
  }

  function plan_update($email, $price_id)
  {
    // POSTする
    $data = [
      'email' => $email,
      'price_id' => $price_id
    ];
    $response = request_post(get_endpoint('planUpdate'), $data);

    // エラー発生時
    if (is_wp_error($response)) {
      $_m = 'APIリクエストが失敗しました(plan_update): ' . $response->get_error_message();
      CalliopeLogger::log($_m);

      return return_response(self::ERROR, '', $_m);
    }

    // レスポンスコード
    $response_code = wp_remote_retrieve_response_code($response);

    // メッセージ
    $body = get_body($response);
    $message_code = get_object_value($body, 'message_code');
    $message = get_object_value($body, 'message');

    if ($response_code === 200) {
      return return_response(self::SUCCESS, $message_code, 'プランを変更しました', $body);
    } else {
      $_m = 'APIリクエストが失敗しました(plan_update): ' . $message . '(' . $response["response"]["message"] . ')';
      CalliopeLogger::log($_m);

      return return_response(self::ERROR, $message_code, $message);
    }
  }

  function plan_cancel($email)
  {
    // POSTする
    $data = [
      'email' => $email,
    ];
    $response = request_post(get_endpoint('planCancel'), $data);

    // エラー発生時
    if (is_wp_error($response)) {
      $_m = 'APIリクエストが失敗しました(plan_cancel): ' . $response->get_error_message();
      CalliopeLogger::log($_m);

      return return_response(self::ERROR, '', $_m);
    }

    // レスポンスコード
    $response_code = wp_remote_retrieve_response_code($response);

    // メッセージ
    $body = get_body($response);
    $message_code = get_object_value($body, 'message_code');
    $message = get_object_value($body, 'message');

    if ($response_code === 200) {
      return return_response(self::SUCCESS, $message_code, 'プランを解約しました', $body);
    } else {
      $_m = 'APIリクエストが失敗しました(plan_cancel): ' . $message . '(' . $response["response"]["message"] . ')';
      CalliopeLogger::log($_m);

      return return_response(self::ERROR, $message_code, $message);
    }
  }
}
