<?php

require_once(dirname(__FILE__) . '/../util.php');
require_once(get_plugin_root() . '/calliope_logger.php');
require_once(get_plugin_root() . '/config.php');
require_once(get_plugin_root() . '/api/base.php');

class UserCertification extends ApiBase
{
  function get_user($uid)
  {
    // POSTする
    $data = [
      'uid' => $uid
    ];
    $response = request_post(get_endpoint('getUser'), $data);

    // エラー発生時
    if (is_wp_error($response)) {
      $_m = 'APIリクエストが失敗しました(get_user): ' . $response->get_error_message();
      CalliopeLogger::log($_m);

      return return_response(self::ERROR, '', $_m);
    }

    // レスポンスコード
    $response_code = wp_remote_retrieve_response_code($response);

    // メッセージ
    $body = get_body($response);
    $message_code = get_object_value($body, 'message_code');
    $message = get_object_value($body, 'message');

    // レスポンスのデータを処理する
    if ($response_code === 200) {
      return return_response(self::SUCCESS, $message_code, 'ユーザー情報を取得しました。', $body);
    } else {
      $_m = 'APIリクエストが失敗しました(get_user): ' . $message . '(' . $response["response"]["message"] . ')';
      CalliopeLogger::log($_m);

      return return_response(self::ERROR, $message_code, $message);
    }
  }

  function user_create($mail, $password)
  {
    // POSTする
    $data = [
      'mail' => $mail,
      'password' => $password
    ];
    $response = request_post(get_endpoint('addUser'), $data);

    // エラー発生時
    if (is_wp_error($response)) {
      $_m = 'APIリクエストが失敗しました(user_create): ' . $response->get_error_message();
      CalliopeLogger::log($_m);

      return return_response(self::ERROR, '', $_m);
    }

    // レスポンスコード
    $response_code = wp_remote_retrieve_response_code($response);

    // メッセージ
    $body = get_body($response);
    $message_code = get_object_value($body, 'message_code');
    $message = get_object_value($body, 'message');

    // レスポンスのデータを処理する
    if ($response_code === 200) {
      return return_response(self::SUCCESS, $message_code, 'ユーザーの登録が完了しました', $body);
    } else {
      $_m = 'APIリクエストが失敗しました(user_create): ' . $message. '('. $response["response"]["message"].')';
      CalliopeLogger::log($_m);

      // 特定のエラーの場合メッセージを置き換える
      if($message == 'The email address is already in use by another account.'){
        $message = 'すでに登録されています。ログイン画面からログインしてください。';
      }elseif($message == 'The password must be a string with at least 6 characters.'){
        $message = 'パスワードは6文字以上で入力してください。';
      }

      return return_response(self::ERROR, $message_code, $message);
    }
  }

  function get_user_id_token_with_email_password($mail, $password)
  {
    // POSTする
    $data = [
      'email' => $mail,
      'password' => $password,
      'returnSecureToken' => true // ないとだめらしい
    ];
    $response = request_post(
      'https://identitytoolkit.googleapis.com/v1/accounts:signInWithPassword?key='.CalliopeConfig::FIREBASE_API_KEY,
      $data
    );

    // エラー発生時
    if (is_wp_error($response)) {
      $_m = 'APIリクエストが失敗しました(get_user_id_token_with_email_password): ' . $response->get_error_message();
      CalliopeLogger::log($_m);

      return return_response(self::ERROR, '', $_m);
    }

    // レスポンスコード
    $response_code = wp_remote_retrieve_response_code($response);

    // メッセージ
    $body = get_body($response);
    $message_code = get_object_value($body, 'message_code');
    $message = get_object_value($body, 'message');

    // レスポンスのデータを処理する
    if ($response_code === 200) {
      return return_response(self::SUCCESS, $message_code, 'idTokenを取得しました。', $body);
    } else {
      // このAPIだけ外部APIなのでエラー処理が特殊
      $error = $body->error;
      $message = $error->message;

      // 特定のエラーの場合メッセージを置き換える
      if ($message == 'INVALID_PASSWORD') {
        $message = 'パスワードが違います。再ログインしてください。';
      } elseif ($message == 'EMAIL_NOT_FOUND') {
        $message = 'メールアドレスが存在しません。再ログインしてください。';
      }

      $_m = 'APIリクエストが失敗しました(get_user_id_token_with_email_password): ' . $message . '(' . $response["response"]["message"] . ')';
      CalliopeLogger::log($error);

      return return_response(self::ERROR, $message_code, $message);
    }
  }

  function user_login($idToken)
  {
    // POSTする
    $data = [
      'idToken' => $idToken,
    ];
    $response = request_post(get_endpoint('loginUser'), $data);

    // エラー発生時
    if (is_wp_error($response)) {
      $_m = 'APIリクエストが失敗しました(user_login): ' . $response->get_error_message();
      CalliopeLogger::log($_m);

      return return_response(self::ERROR, '', $_m);
    }

    // レスポンスコード
    $response_code = wp_remote_retrieve_response_code($response);

    // メッセージ
    $body = get_body($response);
    $message_code = get_object_value($body, 'message_code');
    $message = get_object_value($body, 'message');

    // レスポンスのデータを処理する
    if ($response_code === 200) {
      return return_response(self::SUCCESS, $message_code, 'ログインしました。', $body);
    } else {
      $_m = 'APIリクエストが失敗しました(user_login): ' . $message . '(' . $response["response"]["message"] . ')';
      CalliopeLogger::log($_m);

      return return_response(self::ERROR, $message_code, $message);
    }
  }

  function reset_password($mail)
  {
    // POSTする
    $data = [
      'mail' => $mail,
      'url' => get_accessed_url_host(),
    ];
    $response = request_post(get_endpoint('resetPassword'), $data);

    // エラー発生時
    if (is_wp_error($response)) {
      $_m = 'APIリクエストが失敗しました(reset_password): ' . $response->get_error_message();
      CalliopeLogger::log($_m);

      return return_response(self::ERROR, '', $_m);
    }

    // レスポンスコード
    $response_code = wp_remote_retrieve_response_code($response);

    // メッセージ
    $body = get_body($response);
    $message_code = get_object_value($body, 'message_code');
    $message = get_object_value($body, 'message');

    // レスポンスのデータを処理する
    if ($response_code === 200) {
      return return_response(self::SUCCESS, $message_code, 'パスワードリセットメールを送信しました。', $body);
    } else {
      $_m = 'APIリクエストが失敗しました(reset_password): ' . $message . '(' . $response["response"]["message"] . ')';
      CalliopeLogger::log($_m);

      return return_response(self::ERROR, $message_code, $message);
    }
  }

  function coupon_code_registration($coupon_code)
  {
    global $wpdb;
    // uidを取得する
    $calliope_user = $wpdb->get_row("SELECT * FROM " . CalliopeConfig::TABLE_CALLIOPE_USER . " WHERE id = 1");

    // POSTする
    $data = [
      'uid' => $calliope_user->uid,
      'coupon_code' => $coupon_code,
    ];
    $response = request_post(get_endpoint('couponCodeRegistration'), $data);

    if (is_wp_error($response)) {
      $_m = 'APIリクエストが失敗しました(coupon_code_registration): ' . $response->get_error_message();
      CalliopeLogger::log($_m);

      return return_response(self::ERROR, '', $_m);
    }

    // レスポンスコード
    $response_code = wp_remote_retrieve_response_code($response);

    // メッセージ
    $body = get_body($response);
    $message_code = get_object_value($body, 'message_code');
    $message = get_object_value($body, 'message');

    // レスポンスのデータを処理する
    if ($response_code === 200) {
      return return_response(self::SUCCESS, $message_code, 'クーポンコードの認証が完了しました', $body);
    } else {
      $_m = 'APIリクエストが失敗しました(coupon_code_registration): ' . $message . '(' . $response["response"]["message"] . ')';
      CalliopeLogger::log($_m);

      return return_response(self::ERROR, $message_code, $message);
    }
  }

  function get_expired_date()
  {
    global $wpdb;
    $calliope_user = $wpdb->get_row("SELECT * FROM " . CalliopeConfig::TABLE_CALLIOPE_USER . " WHERE id = 1");

    if (empty($calliope_user)) {
      return return_response(self::ERROR, '', '');
    }

    // POSTする
    $data = [
      'uid' => $calliope_user->uid,
    ];
    $response = request_post(get_endpoint('getExpiredDate'), $data);

    // エラー発生時
    if (is_wp_error($response)) {
      $_m = 'APIリクエストが失敗しました(get_expired_date): ' . $response->get_error_message();
      CalliopeLogger::log($_m);

      return return_response(self::ERROR, $_m);
    }

    // レスポンスコード
    $response_code = wp_remote_retrieve_response_code($response);

    // メッセージ
    $body = get_body($response);
    $message_code = get_object_value($body, 'message_code');
    $message = get_object_value($body, 'message');

    // レスポンスのデータを処理する
    if ($response_code === 200) {
      return return_response(self::SUCCESS, $message_code, '有効期限を取得しました', $body);
    } else {
      $_m = 'APIリクエストが失敗しました(get_expired_date): ' . $message . '(' . $response["response"]["message"] . ')';
      CalliopeLogger::log($_m);

      return return_response(self::ERROR, $message_code, $message);
    }
  }

  function is_expire_date_verified()
  {
    $is_verified = true;
    $_exrs = $this->get_expired_date();
    if ($_exrs['code'] == self::SUCCESS) {
      $expired_date = date('Y/m/d', strtotime($_exrs['data']->expiredDate));
      $now = date('Y/m/d');
      if ($now > $expired_date) {
        $is_verified = false;
      }
    }else{
      $is_verified = false;
    }

    return $is_verified;
  }
}
