<?php

require_once(dirname(__FILE__) . '/../util.php');
require_once(get_plugin_root() . '/calliope_logger.php');
require_once(get_plugin_root() . '/autogenerator/articleformat.php');
require_once(get_plugin_root() . '/autogenerator/prompt.php');
require_once(get_plugin_root() . '/autogenerator/openai/client.php');

# Autogenクラス
class AutoGenerator
{
  public $keyword;
  public $index_count;
  public $feed_related_keyword = '';
  public $custom_prompt = '';
  public $reference_urls = '';
  public $is_view_lead_text;
  public $is_view_index_ad;
  public $is_view_summary;
  public $is_view_last_ad;
  public $embedded_html;

  public $related_keyword = '';
  public $regulation;
  public $index_lists = [];
  public $index_lists_str = '';
  public $index1 = '';
  public $contents1 = '';
  public $index2 = '';
  public $contents2 = '';
  public $index3 = '';
  public $contents3 = '';
  public $index4 = '';
  public $contents4 = '';
  public $index5 = '';
  public $contents5 = '';
  public $summary = '';
  public $lead_text = '';
  public $title = '';

  function __construct(
    $keyword = '筋トレ',
    $feed_related_keyword = '',
    $custom_prompt = '',
    $reference_urls = '',
    $index_count = 5,
    $is_view_lead_text = true,
    $is_view_index_ad = true,
    $is_view_summary = true,
    $is_view_last_ad = true,
    $embedded_html = ''
  ) {
    $this->keyword = $keyword;
    $this->feed_related_keyword = $feed_related_keyword;
    $this->custom_prompt = $custom_prompt;
    $this->reference_urls = $reference_urls;
    $this->index_count = $index_count;
    $this->is_view_lead_text = $is_view_lead_text;
    $this->is_view_index_ad = $is_view_index_ad;
    $this->is_view_summary = $is_view_summary;
    $this->is_view_last_ad = $is_view_last_ad;
    $this->embedded_html = $embedded_html;
  }

  ///// cronなどの外部から実行する場合の初期化関数
  ///// 記事生成に必要なキーワードなどの生成物をキャッシュしておき、この関数から初期化する
  function __init(
    $_related_keyword = '',
    $_regulation = '',
    $_index_lists = [],
    $_index_lists_str = '',
    $_index1 = '',
    $_contents1 = '',
    $_index2 = '',
    $_contents2 = '',
    $_index3 = '',
    $_contents3 = '',
    $_index4 = '',
    $_contents4 = '',
    $_index5 = '',
    $_contents5 = '',
    $_summary = '',
    $_lead_text = '',
    $_title = ''
  ) {
    $this->related_keyword = $_related_keyword;
    $this->regulation = $_regulation;
    $this->index_lists = $_index_lists;
    $this->index_lists_str = $_index_lists_str;
    $this->index1 = $_index1;
    $this->contents1 = $_contents1;
    $this->index2 = $_index2;
    $this->contents2 = $_contents2;
    $this->index3 = $_index3;
    $this->contents3 = $_contents3;
    $this->index4 = $_index4;
    $this->contents4 = $_contents4;
    $this->index5 = $_index5;
    $this->contents5 = $_contents5;
    $this->summary = $_summary;
    $this->lead_text = $_lead_text;
    $this->title = $_title;
  }

  function generate()
  {
    CalliopeLogger::log("----生成開始----");

    $this->generate_related_keyword();

    $this->generate_indexes();

    if ($this->index_count) $this->generate_contents(0);
    if ($this->index_count > 1) $this->generate_contents(1);
    if ($this->index_count > 2) $this->generate_contents(2);
    if ($this->index_count > 3) $this->generate_contents(3);
    if ($this->index_count > 4) $this->generate_contents(4);

    $this->generate_summary();

    $this->generate_lead_text();

    $this->generate_title();


    // 記事生成
    return $this->generate_article();
  }

  //////// 第1段階 : 関連キーワードの抽出
  function generate_related_keyword()
  {
    if (empty($this->feed_related_keyword)) {
      // 設定しているキーワードをそのまま利用する
      $this->related_keyword = $this->keyword;
    } else {
      // 入力している関連キーワードからランダムで取得する
      $_frkw_str = str_replace(array("\r\n", "\r", "\n"), "\n", $this->feed_related_keyword);
      $_frkws = explode("\n", $_frkw_str);

      $_frkws_a = [];
      foreach ($_frkws as $key => $_frkw) {
        // 空文字、改行のみは除去
        if(empty($_frkw)) continue;

        // カンマを空白に置換して配列に格納する
        $_s = str_replace(',', ' ', $_frkw);
        $_s = str_replace('、', ' ', $_frkw);
        $_frkws_a[] = $_s;
      }

      $_k = array_rand($_frkws_a, 1);
      CalliopeLogger::log('randam key is .... '.$_k);
      $random_keyword = $_frkws_a[$_k];
      $this->related_keyword = $this->keyword . ' ' . $random_keyword;
    }

    CalliopeLogger::log('関連キーワード .... ' . $this->related_keyword);
  }

  //////// 第2段階 : レギュレーション
  function generate_regulation()
  {
    $prompt_list = [
      new Prompt($this->prompt_regulation()),
    ];

    $prompt_list = $this->request($prompt_list);
    $response = $prompt_list[count($prompt_list) - 1]->response;

    CalliopeLogger::log("レギュレーション .... " . $response . "");

    $this->regulation = $response;
  }

  //////// 第3段階 : 見出し生成
  function generate_indexes()
  {
    $prompt_list = [
      new Prompt($this->prompt_index()),
    ];

    $prompt_list = $this->request($prompt_list);
    $response = $prompt_list[count($prompt_list) - 1]->response;

    $this->index_lists = get_lists_anser($response);
    $this->index_lists_str = "「" . implode("」,「", $this->index_lists) . "」";

    CalliopeLogger::log("見出し .... ");
    CalliopeLogger::log($this->index_lists_str);
  }

  //////// 第4段階 : 本文生成
  function generate_contents($_idx)
  {
    // 設定されている見出し件数分のみ実行する
    if ($this->index_count <= $_idx) return;

    $prompt_list = [
      new Prompt($this->prompt_content($this->index_lists[$_idx])),
    ];

    $prompt_list = $this->request($prompt_list);
    $response = $prompt_list[count($prompt_list) - 1]->response;

    CalliopeLogger::log("見出し" . $_idx + 1 . " .... " . $this->index_lists[$_idx] . "");
    CalliopeLogger::log("本文 .... " . $response . "");

    $_index_var = 'index' . ($_idx + 1);
    $_contents_var = 'contents' . ($_idx + 1);

    $this->{$_index_var} = $this->index_lists[$_idx];
    $this->{$_contents_var} = $response;
  }

  //////// 第5段階 : 要約生成
  function generate_summary()
  {
    $prompt_list = [
      new Prompt($this->prompt_summary()),
    ];

    $prompt_list = $this->request($prompt_list);
    $response = $prompt_list[count($prompt_list) - 1]->response;

    CalliopeLogger::log("要約 .... " . $response . "");

    $this->summary = $response;
  }

  //////// 第6段階 : リード文生成
  function generate_lead_text()
  {
    $prompt_list = [
      new Prompt($this->prompt_lead_text()),
    ];

    $prompt_list = $this->request($prompt_list);
    $response = $prompt_list[count($prompt_list) - 1]->response;

    CalliopeLogger::log("リード文 .... " . $response . "");

    $this->lead_text = $response;
  }

  //////// 第7段階 : タイトル生成
  function generate_title()
  {
    $prompt_list = [
      new Prompt($this->prompt_title()),
    ];

    $prompt_list = $this->request($prompt_list);
    $response = $prompt_list[count($prompt_list) - 1]->response;

    CalliopeLogger::log("タイトル .... " . $response . "");

    $this->title = $response;

    // タイトルが「」でくくられている場合があるので、除去する
    if (mb_substr($this->title, 0, 1) === '「' && mb_substr($this->title, -1) === '」') {
      $this->title = mb_substr($this->title, 1);
      $this->title = mb_substr($this->title, 0, -1);
    }
  }

  //////// 第8段階 : パーマリンク生成
  function generate_permalink()
  {
    $prompt_list = [
      new Prompt($this->prompt_permalink()),
    ];

    $prompt_list = $this->request($prompt_list);
    $response = $prompt_list[count($prompt_list) - 1]->response;

    CalliopeLogger::log("パーマリンク .... " . $response . "");

    return $response;
  }

  //////// 第9段階 : 記事の生成
  function generate_article()
  {
    $lead_text_arr = [];
    $index_contents_arr = [];
    $summary_arr = [];

    // リード文を表示する
    if ($this->is_view_lead_text) {
      $lead_text_arr[] = new ArticleFormat($this->lead_text, "", true);
    }

    // 見出し+本文を表示する（広告を表示する場合は併せて表示）
    for ($i = 0; $i < 5; $i++) {
      $this->add_index_contents_arr($i, $index_contents_arr);
    }

    // 要約を表示する
    if ($this->is_view_summary) {
      $summary_arr[] = new ArticleFormat("まとめ", "h2");
      $summary_arr[] = new ArticleFormat($this->summary, "", true);
    }

    // 文末広告を表示する
    if ($this->is_view_last_ad) {
      $summary_arr[] = new ArticleFormat($this->embedded_html, "");
    }

    $article_arr = array_merge($lead_text_arr, $index_contents_arr, $summary_arr);

    $contents_str = '';
    foreach ($article_arr as $article) {
      $contents_str .= $article->get_html_format();
    }

    return [$this->title, $contents_str];
  }

  function add_index_contents_arr($_idx, &$index_contents_arr)
  {
    if ($this->index_count > $_idx) {
      $_idx_var = 'index' . ($_idx + 1);
      $_cts_var = 'contents' . ($_idx + 1);
      $index_contents_arr[] = new ArticleFormat($this->{$_idx_var}, "h2");
      $index_contents_arr[] = new ArticleFormat($this->{$_cts_var}, "", true);

      // 見出し語と広告を挿入する
      if ($this->is_view_index_ad) {
        $index_contents_arr[] = new ArticleFormat($this->embedded_html, "");
      }
    }
  }

  function request($prompt_list)
  {
    $client = new Client;
    $messages_array = [];

    foreach ($prompt_list as $article_format) {
      $try_cnt = 0;

      while ($try_cnt < 10) {
        # messages_arrayにrole:user、リクエスト文を追加する。
        $messages_array[] = [
          "role" => "user",
          "content" => $article_format->prompt
        ];
        # messages_arrayでリクエストする。
        $res = $client->completions($messages_array);

        if (empty($res)) {
          // 最新のリクエストを削除する
          array_pop($messages_array);

          CalliopeLogger::log("リクエストに失敗しました。もう一度やります。");
        } else {
          $article_format->response = $res;

          # リクエストした内容をrole:systemにする。
          $messages_array[count($messages_array) - 1]["role"] = "system";
          # レスポンス内容をrole:assistantとしてmessages_arrayににする。
          $messages_array[] = [
            "role" => "assistant",
            "content" => $article_format->response
          ];

          break;
        }

        $try_cnt++;
      }
    }

    return $prompt_list;
  }

  //// 共通プロンプト
  function common_prompt()
  {
    global $wpdb;

    $urls = '';
    $calliope_setting = $wpdb->get_row("SELECT * FROM " . CalliopeConfig::TABLE_CALLIOPE_SETTING . " WHERE id = 1");
    if($calliope_setting->gpt_version != CalliopeConfig::CALLIOPE_GPT3){
      $urls = <<< EOM
============================================================
■ 参考URL
============================================================
{$this->format_urls()}
EOM;
    }

    return
      <<< EOM
あなたはSEOを十分に理解してる優秀な日本人のWEBライターです。
「{$this->related_keyword}」というキーワードでブログ記事を作成し、Google検索の上位10件に入る記事を書くことを目的とします。
上記の目的において、以下の「基本ルール」「この記事の情報と基本方針」「ルールと依頼内容」を遵守して依頼を遂行してください。

============================================================
■ 基本ルール
============================================================
・Google検索で評価される内容にしてください。
・炎上リスクがない、攻撃的な内容ではない、法律に抵触しない記事内容を心がけてください。
・「成功」「鍵」などのAIが生成したと思われる単語の使用を避けてください
・設定しているキーワードを適切に内容に含め、SEO観点で上位表示を狙う記事内容にしてください。
{$this->format_prompt()}

{$urls}

============================================================
■ この記事の情報と基本方針
============================================================
{$this->regulation}
EOM;
  }

  function format_urls(){
    $_fru_str = str_replace(array("\r\n", "\r", "\n"), "\n", $this->reference_urls);
    $_frus = explode("\n", $_fru_str);

    $_frus_a = [];
    foreach ($_frus as $key => $_fru) {
      // 空文字、改行のみは除去
      if (empty($_fru)) continue;

      // フォーマット
      $_frus_a[] = $_fru;
    }

    return implode("\n", $_frus_a);
  }

  function format_prompt()
  {
    $_frcp_str = str_replace(array("\r\n", "\r", "\n"), "\n", $this->custom_prompt);
    $_frcps = explode("\n", $_frcp_str);

    $_frcps_a = [];
    foreach ($_frcps as $key => $_frcp) {
      // 空文字、改行のみは除去
      if (empty($_frcp)) continue;

      // フォーマット
      $_s = '・' . $_frcp;
      $_frcps_a[] = $_s;
    }

    return implode("\n", $_frcps_a);
  }

  //// レギュレーションプロンプト
  function prompt_regulation()
  {
    return
      <<< EOM
あなたはSEOを十分に理解してる優秀な日本人のWEBライターです。
「{$this->related_keyword}」というキーワードでブログ記事を作成し、Google検索の上位10件に入る記事を書くことを目的とします。

その上で、このキーワードで記事を書く際のレギュレーションを以下のフォーマットに従って生成してもらいたいです。
各項目の文字数は「：」の右側に指定しているので、遵守してください。

「この記事の基本方針」の「導入」「本文」「まとめ」の欄には、この記事を完成させるうえでの基本方針を、指定した文字数程度で考えてもらいたいです。

■ フォーマット
キーワード：
この記事の想定ターゲット：100文字程度
読者が何に悩んでいるか？：100文字程度
この記事の基本方針
	1. 導入：200文字程度
	2. 本文：200文字程度
	3. まとめ：200文字程度
EOM;
  }

  function prompt_index()
  {
    return
      <<< EOM
{$this->common_prompt()}

============================================================
■ ルールと依頼内容
============================================================
・「{$this->related_keyword}」というキーワードで検索してくるユーザーが最も知りたいことや悩んでいることを解決する目次を、ブログのプロの目線で{$this->index_count}個作ってください。
・見出しは大見出しのみで作成してください。小見出しは不要です。各見出しの詳細は不要です。
・見出しにはGoogleに評価されるように、キーワードを含めつつ20文字程度でシンプルにまとめたものを生成してください。
・この記事を読むターゲットが何に悩んでいるかを考慮し、この記事の基本方針に沿って見出しを生成するようにしてください。
・見出しを作る際は、起承転結でユーザーの悩みを解決するステップを踏むようにしてください。
・なるべく見出しごとに同じような内容にならないように注意してください。

============================================================
■ 禁止事項
============================================================
※以下の禁止事項を必ず守ってください
・""や「」などを使用しないでください
EOM;
  }

  function prompt_content($index)
  {
    return
      <<< EOM
{$this->common_prompt()}

============================================================
■ ルールと依頼内容
============================================================
・{$this->index_lists_str}という見出しで記事を作成中です。
・今回は「{$index}」という見出しに関して、ブログのプロの目線でわかりやすい文章を700文字以上で書いてください。
・以下の「現時点の記事情報」に作成中の記事情報を記載しています。この内容と同じ内容を生成しないように注意してください。
・「この記事の情報と基本方針」の「本文」に記載された内容を参考に文章を考えてください。
・ユーザーが感じている課題感や問題点を解決できるような記事内容にしてください。
・専門用語を使用する場合は、その用語について簡単な説明を含めてください。
・記事内容はhtmlで生成してください。また、以下のタグは不要です。
    1. <html>タグ
    2. <head>タグ
    3. <body>タグ
    4. <h1><h2>タグ
・内容を分割する必要がある場合は適切に<h3>と<h4>タグを使用し、その内容に沿った記事内容にしてください。
・読みやすさを重視し、適切なチャンクに区切ったり、改行を含めるようにしてください。
・改行には<br>タグを使用してください。
・ブログを分かりやすくするために、必要であれば下記の記載方法を使ってください。使用しなくても構いません。
    1.特に伝えたいポイントは<strong></strong>で囲んでください。
    2.リストを作りたい場合は
    <ul class="clp-l-ul">
      <li class="clp-l-li">これはダミーのテキストです</li>
      <li class="clp-l-li">これはダミーのテキストです</li>
    </ul>
    で作成してください。

============================================================
■ 禁止事項
============================================================
※以下の禁止事項を必ず守ってください
・見出しをそのまま記事内に表示するのはやめてください。
・まとめは不要なので作らないでください。
・「ブログのプロの目線で」などの文言は含まないようにしてください。あくまでこの記事を読む読者に向けての内容にしてください。
・markdownは使用しないでください。

============================================================
■ 現時点の記事情報
============================================================
{$this->contents1}
{$this->contents2}
{$this->contents3}
{$this->contents4}
{$this->contents5}
EOM;
  }

  function prompt_summary()
  {
    return
      <<< EOM
{$this->common_prompt()}

============================================================
■ ルールと依頼内容
============================================================
・「{$this->related_keyword}」というキーワードと{$this->index_lists_str}という見出しでブログの記事を書いています。
・この内容を踏まえて、ブログの文章の最後に掲載するための文章を200文字以内にまとめ、ブログのプロの目線でわかりやすく作ってください。
・「この記事の情報と基本方針」の「まとめ」に記載された内容を参考に文章を考えてください。
EOM;
  }

  function prompt_lead_text()
  {
    return
      <<< EOM
{$this->common_prompt()}

============================================================
■ ルールと依頼内容
============================================================
・「{$this->related_keyword}」というキーワードと{$this->index_lists_str}という見出しでブログの記事を書いています。
・この内容を踏まえて、検索ユーザーが共感し、記事を読む気持ちが高まる魅力的なリード文（導入文）を200文字以内で作ってください。
・ユーザーへの共感と問題提起を含む内容を心がけてください
・「この記事の情報と基本方針」の「導入」に記載された内容を参考に文章を考えてください。
EOM;
  }

  function prompt_title()
  {
    return
      <<< EOM
{$this->common_prompt()}

============================================================
■ ルールと依頼内容
============================================================
・「{$this->related_keyword}」というキーワードと{$this->index_lists_str}という見出しでブログの記事を書いています。
・この内容を踏まえて、検索ユーザーが興味を持ちクリックする確率が高そうな魅力的な記事のタイトルを30文字程度で1つ作ってください。
・この記事の想定ターゲットが思わずクリックしたくなるような内容を考えてください。

============================================================
■ 禁止事項
============================================================
※以下の禁止事項を必ず守ってください
・""や「」などを使用しないでください
・タイトル以外の文字列の生成は不要です。
EOM;
  }

  function prompt_permalink()
  {
    return
      <<< EOM
・「{$this->title}」というタイトルでブログの記事を書いています。
・この内容を踏まえて、20文字程度のパーマリンクを生成してください。
・パーマリンク以外の文字列は不要です。
・小文字の英語、ハイフンのみで生成してください。
EOM;
  }
}
