<?php

require_once(dirname(__FILE__) . '/../../util.php');
require_once(get_plugin_root() . '/config.php');

class Client
{
    private $API_KEY = '';
    private $GPT_VERSION = '';

    function __construct($api_key = null, $gpt_version = null)
    {
        global $wpdb;
        $calliope_setting = $wpdb->get_row("SELECT * FROM " . CalliopeConfig::TABLE_CALLIOPE_SETTING . " WHERE id = 1");

        if ($gpt_version == null) {
            $this->GPT_VERSION = $calliope_setting->gpt_version;
        } else {
            $this->GPT_VERSION = $gpt_version;
        }

        if ($api_key == null) {
            $this->API_KEY = $calliope_setting->open_ai_api_key;
        } else {
            $this->API_KEY = $api_key;
        }
    }

    public function completions($words)
    {
        $url = "https://api.openai.com/v1/chat/completions";

        // リクエストヘッダー
        $headers = [
            'Content-Type: application/json',
            'Authorization: Bearer ' . $this->API_KEY
        ];

        // リクエストボディ
        $data = [
            'model' => $this->GPT_VERSION,
            'messages' => $words,
            'max_tokens' => 2048,
            'temperature' => 0.5
        ];

        // cURLを使用してAPIにリクエストを送信
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($data));
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);

        $response = curl_exec($ch);
        if (curl_errno($ch)) {
            CalliopeLogger::log('OpenAI API利用時にエラーが発生しました :::::: '.curl_error($ch));
        }
        curl_close($ch);

        // 結果をデコード
        $result = json_decode($response, true);

        if(isset($result['error'])){
            CalliopeLogger::log('OpenAI APIの返却値にエラーが発生しました :::::: ' . $result['error']['message']);
        }

        $res = '';
        if(isset($result['choices'][0]['message']['content'])){
            $res = str_replace('/n', '', $result['choices'][0]['message']['content']);
        }

        return $res;
    }
}
