<?php
/*
Plugin Name: Calliope
Plugin URI: プラグインの公式サイトの案内用の自由なURL
Description: Wordpress AI Contents Generator
Version: 2.6.0
Author: homio
*/

require_once(dirname(__FILE__) . '/util.php');
require_once(get_plugin_root() . '/menu.php');
require_once(get_plugin_root() . '/config.php');
require_once(get_plugin_root() . '/calliope_logger.php');
require_once(get_plugin_root() . '/scheduler.php');
require_once(get_plugin_root() . '/db/seeds.php');
require_once(get_plugin_root() . '/db/migration.php');
require_once(get_plugin_root() . '/updater/plugin-update-checker.php');

use YahnisElsts\PluginUpdateChecker\v5\PucFactory;

///// 更新の確認
$ExampleUpdateChecker = PucFactory::buildUpdateChecker(
  'https://calliope.devomio.work/calliope-update-checker.json',
  __FILE__,
  'Calliope'
);


///// プラグイン有効化時
register_activation_hook(__FILE__, 'plugin_activation');
function plugin_activation()
{
  CalliopeLogger::log('Calliope activation....');

  // マイグレーションの実行
  calliope_migration();

  // cronの登録
  if (!wp_next_scheduled('calliope_cron_scheduler')) {
    wp_schedule_event(time(), 'every_ten_minutes', 'calliope_cron_scheduler');
  }
}

///// プラグイン無効化時
register_deactivation_hook(__FILE__, 'plugin_deactivation');
function plugin_deactivation()
{
  global $wpdb;

  // サブスクリプションの解約
  $calliope_user = $wpdb->get_row("SELECT * FROM " . CalliopeConfig::TABLE_CALLIOPE_USER . " WHERE id = 1");
  if (!empty($calliope_user)) {
    $email = $calliope_user->mail;

    $stripe = new Stripe();
    $stripe->plan_cancel($email);
  }

  // テーブルの削除
  delete_tables();

  // cronの削除
  wp_clear_scheduled_hook('calliope_cron_scheduler');
}

///// プラグインアップデート時
add_action('upgrader_process_complete', 'calliope_plugins_updated');
function calliope_plugins_updated()
{
  calliope_migration();
}

///// プラグイン読み込み時
add_action('init', 'calliope_plugins_loaded');
function calliope_plugins_loaded()
{
  /**
   * あんまりやりたくないけど、アップデートしてもマイグレーションがちゃんと走らない不具合があるので
   * プラグイン読み込み時に毎回マイグレーションを実行することにする
   */
  // マイグレーションの実行
  calliope_migration();

  /**
   * あんまりやりたくないけど、サイトの引っ越しとかでregister_activation_hookが呼ばれないことがあるので、
   * その時用にcronの登録状況を確認して、無ければ登録する処理を実行する
   */
  // cronの登録
  if (!wp_next_scheduled('calliope_cron_scheduler')) {
    wp_schedule_event(time(), 'every_ten_minutes', 'calliope_cron_scheduler');
  }
}


///// プラグインの自動更新設定を促す
// Calliopeの自動更新がOFFの場合、管理画面上にノーティスを表示させる
// Calliopeのページを開いたときはポップアップを表示させる
function check_specific_plugin_auto_update_status()
{
  // 自動更新が有効になっているプラグインのリストを取得
  $auto_update_plugins = get_option('auto_update_plugins', array());

  // 対象のプラグインスラグ。プラグインフォルダ名/プラグインファイル名.php
  $plugin_slug = 'calliope/calliope.php';

  if (!in_array($plugin_slug, $auto_update_plugins)) {
    // 自動更新が無効になっている場合の処理
    add_action('admin_notices', 'show_auto_update_disabled_notice_for_specific_plugin');
  }
}
add_action('admin_init', 'check_specific_plugin_auto_update_status');

function show_auto_update_disabled_notice_for_specific_plugin()
{
  $html = '<div id="auto-update-notice" class="notice notice-warning is-dismissible">
            <p><strong>Calliopeの自動更新が無効になっています。</strong></p>
            <p><strong>セキュリティを確保するため、「<a href="/wp-admin/plugins.php">プラグイン</a>」から自動更新を有効にしてください。</strong></p>
           </div>';

  echo $html;

  // calliope管理画面のみポップアップを表示する
  if (isset($_GET['page']) && strpos($_GET['page'], 'calliope') !== false) {
    add_action('admin_footer', 'add_auto_update_popup_script');
  }
}

function add_auto_update_popup_script()
{
?>
  <div id="customPopup" style="display:none; position:fixed; left:0; top:0; width:100%; height:100%; background-color:rgba(0,0,0,0.5); z-index:1000;">
    <div style="background-color:white; width:300px; margin:100px auto; padding:20px; border-radius:8px; box-shadow:0 0 10px rgba(0,0,0,0.5);">
      <p><strong>Calliopeの自動更新が無効になっています。</strong></p>
      <p><strong>設定ページで自動更新を有効にしましょう！</strong></p>
      <div style="display:flex; justify-content: space-between;">
        <div></div>
        <button id="okButton" class="button-primary">OK</button>
      </div>
    </div>
  </div>

  <script type="text/javascript">
    document.addEventListener('DOMContentLoaded', function() {
      var popup = document.getElementById('customPopup');
      var okButton = document.getElementById('okButton');

      // OKボタンをクリックしたときの処理
      okButton.addEventListener('click', function() {
        window.location.href = '<?php echo admin_url('plugins.php'); ?>'; // ここに遷移先のURLを設定
      });

      // ポップアップを表示
      popup.style.display = 'block';
    });
  </script>
<?php
}


///// cronの実行間隔を登録するフィルター
add_filter('cron_schedules', 'add_calliope_cron_interval');
function add_calliope_cron_interval($schedules)
{
  $schedules['every_ten_minutes'] = array(
    'interval'  => 600,
    'display'   => __('10分ごと', 'textdomain')
  );
  return $schedules;
}

///// cronを実行するアクションフック
///// CalliopeSchedulerクラスのコンストラクタで、cronで使用するactionをすべて登録する
$cs = new CalliopeScheduler();
add_action('calliope_cron_scheduler', [$cs, 'run']);


///// メニューの追加
add_menus();


///// cssの読み込み
function enqueue_admin_styles()
{
  wp_enqueue_style('my-admin-styles', plugins_url('css/style.css', __FILE__));
}
add_action('admin_enqueue_scripts', 'enqueue_admin_styles');
