<?php

require_once(dirname(__FILE__) . '/../util.php');
require_once(get_plugin_root() . '/config.php');
require_once(get_plugin_root() . '/calliope_logger.php');
require_once(get_plugin_root() . '/db/seeds.php');

function calliope_migration(){
  // マイグレーションテーブルを作成
  mig_migration_table();

  /// ver1.0.0
  if (!is_exist_migrate('1.0.0')) mig_1_0_0();
  /// ver1.0.3
  if (!is_exist_migrate('1.0.3')) mig_1_0_3();
  /// ver1.3.0
  if (!is_exist_migrate('1.3.0')) mig_1_3_0();
  /// ver1.8.0
  if (!is_exist_migrate('1.8.0')) mig_1_8_0();
  /// ver2.1.0
  if (!is_exist_migrate('2.1.0')) mig_2_1_0();
  /// ver2.2.0
  if (!is_exist_migrate('2.2.0')) mig_2_2_0();
  /// ver2.3.0
  if (!is_exist_migrate('2.3.0')) mig_2_3_0();
  /// ver2.4.0
  if (!is_exist_migrate('2.4.0')) mig_2_4_0();
  /// ver2.4.1
  if (!is_exist_migrate('2.4.1')) mig_2_4_1();
  /// ver2.5.1
  if (!is_exist_migrate('2.5.1')) mig_2_5_1();
  /// ver2.6.0
  if (!is_exist_migrate('2.6.0')) mig_2_6_0();
}

function compare_versions($current_version, $target_version)
{
  // バージョンをドットで分割し、配列に格納する
  $current_version_parts = explode('.', $current_version);
  $target_version_parts = explode('.', $target_version);

  // 数値に変換してバージョン番号を比較する
  for ($i = 0; $i < 3; $i++) {
    $current_num = isset($current_version_parts[$i]) ? intval($current_version_parts[$i]) : 0;
    $target_num = isset($target_version_parts[$i]) ? intval($target_version_parts[$i]) : 0;

    if ($current_num < $target_num) {
      return -1; // 現在のバージョンが対象バージョンよりも古い
    } elseif ($current_num > $target_num) {
      return 1; // 現在のバージョンが対象バージョンよりも新しい
    }
  }

  return 0; // バージョンが同じ
}

// マイグレーションの履歴を残す
function migrate_history_update($ver){
  DBoperation::db_insert(
    CalliopeConfig::TABLE_CALLIOPE_MIGRATE,
    [
      'migrate' => $ver
    ]
  );
}

// 指定バージョンのマイグレーションが実行済みかを判定する
function is_exist_migrate($ver){
  global $wpdb;

  $mig_row = $wpdb->get_row("SELECT * FROM " . CalliopeConfig::TABLE_CALLIOPE_MIGRATE . " WHERE migrate = '". $ver. "'");

  if(!empty($mig_row)){
    return true;
  }
}

////////////////////////////////////////////////////////////
/// マイグレーション
////////////////////////////////////////////////////////////

/// マイグレーションテーブルを作成
function mig_migration_table()
{
  global $wpdb;

  $sql = "CREATE TABLE IF NOT EXISTS `" . CalliopeConfig::TABLE_CALLIOPE_MIGRATE . "` (
            `id` int(11) unsigned NOT NULL AUTO_INCREMENT,
            `migrate` text,
            `created_at` timestamp NULL DEFAULT '0000-00-00 00:00:00',
            `modified_at` timestamp NULL DEFAULT '0000-00-00 00:00:00',
            PRIMARY KEY (`id`)
          ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;";

  $wpdb->query($sql);
}

/// ver1.0.0
function mig_1_0_0()
{
  CalliopeLogger::log('migrate ver1.0.0 start .... ');

  // イニシャルのテーブル作成
  create_tables();

  migrate_history_update('1.0.0');

  CalliopeLogger::log('migrate ver1.0.0 completed .... ');
}

/// ver1.0.3
function mig_1_0_3(){
  CalliopeLogger::log('migrate ver1.0.3 start .... ');

  global $wpdb;

  $sql = "ALTER TABLE " . CalliopeConfig::TABLE_CALLIOPE_FEEDS . " MODIFY post_type text";
  $wpdb->query($sql);

  migrate_history_update('1.0.3');

  CalliopeLogger::log('migrate ver1.0.3 completed .... ');
}

/// ver1.3.0
function mig_1_3_0()
{
  CalliopeLogger::log('migrate ver1.3.0 start .... ');

  global $wpdb;

  $sql = "ALTER TABLE " . CalliopeConfig::TABLE_CALLIOPE_USER . "
          CHANGE is_token_verified is_coupon_code_registration tinyint(1) DEFAULT NULL COMMENT 'クーポンコード登録フラグ'";

  $wpdb->query($sql);

  migrate_history_update('1.3.0');

  CalliopeLogger::log('migrate ver1.3.0 completed .... ');
}

/// ver1.8.0
function mig_1_8_0()
{
  CalliopeLogger::log('migrate ver1.8.0 start .... ');

  global $wpdb;

  $sql = "ALTER TABLE " . CalliopeConfig::TABLE_CALLIOPE_FEEDS . "
          MODIFY `name` varchar(255) NOT NULL DEFAULT '' COMMENT 'フィードの名前'";

  $wpdb->query($sql);

  migrate_history_update('1.8.0');

  CalliopeLogger::log('migrate ver1.8.0 completed .... ');
}

/// ver2.1.0
function mig_2_1_0()
{
  CalliopeLogger::log('migrate ver2.1.0 start .... ');

  global $wpdb;

  $sql = "ALTER TABLE " . CalliopeConfig::TABLE_CALLIOPE_SETTING . "
          ADD `gpt_version` varchar(255) NOT NULL DEFAULT 'gpt-3.5-turbo' COMMENT 'GPTのバージョン' AFTER `is_cron_exec`";

  $wpdb->query($sql);

  migrate_history_update('2.1.0');

  CalliopeLogger::log('migrate ver2.1.0 completed .... ');
}

/// ver2.2.0
function mig_2_2_0()
{
  CalliopeLogger::log('migrate ver2.2.0 start .... ');

  global $wpdb;

  $sql = "ALTER TABLE " . CalliopeConfig::TABLE_CALLIOPE_FEEDS_SCHEDULE . "
          ADD `regulation` text COMMENT 'レギュレーション' AFTER `related_keyword`";

  $wpdb->query($sql);

  migrate_history_update('2.2.0');

  CalliopeLogger::log('migrate ver2.2.0 completed .... ');
}

/// ver2.3.0
function mig_2_3_0()
{
  CalliopeLogger::log('migrate ver2.3.0 start .... ');

  global $wpdb;

  $sql = "ALTER TABLE " . CalliopeConfig::TABLE_CALLIOPE_FEEDS . "
          ADD `custom_prompt` text COMMENT 'カスタムプロンプト' AFTER `related_keywords`";

  $wpdb->query($sql);

  $sql = "ALTER TABLE " . CalliopeConfig::TABLE_CALLIOPE_FEEDS . "
          ADD `reference_urls` text COMMENT '参考URL' AFTER `custom_prompt`";

  $wpdb->query($sql);

  migrate_history_update('2.3.0');

  CalliopeLogger::log('migrate ver2.3.0 completed .... ');
}

/// ver2.4.0
function mig_2_4_0()
{
  CalliopeLogger::log('migrate ver2.4.0 start .... ');

  global $wpdb;

  $sql = "ALTER TABLE " . CalliopeConfig::TABLE_CALLIOPE_FEEDS . "
          CHANGE ad_tag embedded_html text COMMENT '埋め込みHTML'";

  $wpdb->query($sql);

  migrate_history_update('2.4.0');

  CalliopeLogger::log('migrate ver2.4.0 completed .... ');
}

/// ver2.4.0
function mig_2_4_1()
{
  CalliopeLogger::log('migrate ver2.4.1 start .... ');

  global $wpdb;

  $sql = "UPDATE " . CalliopeConfig::TABLE_CALLIOPE_FEEDS . "
          SET is_view_index_ad = 0";

  $wpdb->query($sql);

  migrate_history_update('2.4.1');

  CalliopeLogger::log('migrate ver2.4.1 completed .... ');
}

/// ver2.5.1
/// なぜか実行されてないサイトがあったので再実行
function mig_2_5_1()
{
  CalliopeLogger::log('migrate ver2.5.1 start .... ');

  global $wpdb;

  $sql = "ALTER TABLE " . CalliopeConfig::TABLE_CALLIOPE_FEEDS . " MODIFY post_type text";
  $wpdb->query($sql);

  migrate_history_update('2.5.1');

  CalliopeLogger::log('migrate ver2.5.1 completed .... ');
}

/// ver2.6.0
function mig_2_6_0()
{
  CalliopeLogger::log('migrate ver2.6.0 start .... ');

  global $wpdb;

  $sql = "ALTER TABLE " . CalliopeConfig::TABLE_CALLIOPE_FEEDS . "
          CHANGE theme keyword varchar(255) NOT NULL DEFAULT '' COMMENT 'キーワード'";

  $wpdb->query($sql);

  migrate_history_update('2.6.0');

  CalliopeLogger::log('migrate ver2.6.0 completed .... ');
}