<?php

require_once(dirname(__FILE__) . '/../util.php');
require_once(get_plugin_root() . '/config.php');

function create_tables()
{
  create_calliope_feeds_table();
  create_calliope_setting_table();
  create_calliope_feeds_schedule_table();
  create_calliope_user_table();
}

function delete_tables()
{
  global $wpdb;

  $wpdb->query("DROP TABLE IF EXISTS " . CalliopeConfig::TABLE_CALLIOPE_FEEDS);
  $wpdb->query("DROP TABLE IF EXISTS " . CalliopeConfig::TABLE_CALLIOPE_SETTING);
  $wpdb->query("DROP TABLE IF EXISTS " . CalliopeConfig::TABLE_CALLIOPE_FEEDS_SCHEDULE);
  $wpdb->query("DROP TABLE IF EXISTS " . CalliopeConfig::TABLE_CALLIOPE_USER);
  $wpdb->query("DROP TABLE IF EXISTS " . CalliopeConfig::TABLE_CALLIOPE_MIGRATE);
}

function create_calliope_feeds_table()
{
  $sql = "CREATE TABLE `" . CalliopeConfig::TABLE_CALLIOPE_FEEDS . "` (
          `id` int(11) unsigned NOT NULL AUTO_INCREMENT COMMENT 'ID',
          `name` varchar(11) NOT NULL DEFAULT '' COMMENT 'フィードの名前',
          `theme` varchar(255) NOT NULL DEFAULT '' COMMENT 'テーマ',
          `related_keywords` text COMMENT '関連キーワード',
          `cron_interval` int(11) NOT NULL DEFAULT '0' COMMENT 'スケジュール',
          `is_view_lead_text` tinyint(1) NOT NULL DEFAULT '1' COMMENT 'リードテキスト表示',
          `index_count` tinyint(1) NOT NULL DEFAULT '1' COMMENT '見出し数',
          `is_view_index_ad` tinyint(1) NOT NULL DEFAULT '1' COMMENT '見出しごと広告',
          `is_view_summary` tinyint(1) NOT NULL DEFAULT '1' COMMENT '要約表示',
          `is_view_last_ad` tinyint(1) NOT NULL DEFAULT '1' COMMENT '文末広告',
          `ad_tag` text COMMENT '広告タグ',
          `status` tinyint(1) NOT NULL DEFAULT '0' COMMENT '投稿ステータス',
          `post_type` tinyint(1) DEFAULT NULL COMMENT '投稿タイプ',
          `category` text COMMENT 'カテゴリ',
          `tag` text COMMENT 'タグ',
          `last_cron_executed` timestamp NULL DEFAULT NULL COMMENT 'cronの最終実行時間',
          `created_at` timestamp NULL DEFAULT 0,
          `modified_at` timestamp NULL DEFAULT 0,
          PRIMARY KEY (`id`)
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8;";

  require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
  dbDelta($sql);
}

function create_calliope_setting_table()
{
  $sql = "CREATE TABLE `" . CalliopeConfig::TABLE_CALLIOPE_SETTING . "` (
          `id` int(11) unsigned NOT NULL AUTO_INCREMENT,
          `open_ai_api_key` varchar(256) DEFAULT NULL COMMENT 'OpenAIのAPIキー',
          `is_cron_exec` tinyint(1) DEFAULT '0' COMMENT '自動実行フラグ',
          `created_at` timestamp NULL DEFAULT 0,
          `modified_at` timestamp NULL DEFAULT 0,
          PRIMARY KEY (`id`)
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8;";

  require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
  dbDelta($sql);
}

function create_calliope_feeds_schedule_table()
{
  $sql = "CREATE TABLE `" . CalliopeConfig::TABLE_CALLIOPE_FEEDS_SCHEDULE . "` (
          `id` int(11) unsigned NOT NULL AUTO_INCREMENT,
          `calliope_feeds_id` int(11) DEFAULT NULL COMMENT 'calliope_feedsテーブルのID',
          `post_id` int(11) DEFAULT NULL COMMENT 'postsテーブルのID',
          `status` varchar(256) DEFAULT NULL COMMENT '実行状況',
          `related_keyword` text COMMENT '関連キーワード',
          `index_lists` text COMMENT '見出しリスト',
          `index_lists_str` text COMMENT '見出しリスト（文字列連結）',
          `index1` text COMMENT '見出し1',
          `contents1` text COMMENT 'コンテンツ1',
          `index2` text COMMENT '見出し2',
          `contents2` text COMMENT 'コンテンツ2',
          `index3` text COMMENT '見出し3',
          `contents3` text COMMENT 'コンテンツ3',
          `index4` text COMMENT '見出し4',
          `contents4` text COMMENT 'コンテンツ4',
          `index5` text COMMENT '見出し5',
          `contents5` text COMMENT 'コンテンツ5',
          `lead_text` text COMMENT 'リード文',
          `summary` text COMMENT '要約',
          `title` text COMMENT 'タイトル',
          `created_at` timestamp NULL DEFAULT 0,
          `modified_at` timestamp NULL DEFAULT 0,
          PRIMARY KEY (`id`)
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8;";

  require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
  dbDelta($sql);
}

function create_calliope_user_table()
{
  $sql = "CREATE TABLE `" . CalliopeConfig::TABLE_CALLIOPE_USER . "` (
          `id` int(11) unsigned NOT NULL AUTO_INCREMENT,
          `mail` varchar(255) DEFAULT NULL COMMENT 'メールアドレス',
          `uid` varchar(255) DEFAULT NULL COMMENT 'firebaseのuid',
          `is_user_created` tinyint(1) DEFAULT NULL COMMENT 'ユーザー登録フラグ',
          `is_token_verified` tinyint(1) DEFAULT NULL COMMENT 'クーポンコード登録フラグ',
          `created_at` timestamp NULL DEFAULT 0,
          `modified_at` timestamp NULL DEFAULT 0,
          PRIMARY KEY (`id`)
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8;";

  require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
  dbDelta($sql);
}
