<?php

require_once(dirname(__FILE__) . '/util.php');
require_once(get_plugin_root() . '/calliope_logger.php');
require_once(get_plugin_root() . '/api/user_certification.php');
require_once(get_plugin_root() . '/autogenerator/autogenerator.php');
require_once(get_plugin_root() . '/db/operation.php');

class CalliopeScheduler
{
  public $is_test_exec = false;

  function __construct($_is_test_exec = false)
  {
    $this->add_single_action('cron_generate_related_keyword');
    $this->add_single_action('cron_generate_regulation');
    $this->add_single_action('cron_generate_indexes');
    $this->add_single_action('cron_generate_contents');
    $this->add_single_action('cron_generate_summary');
    $this->add_single_action('cron_generate_lead_text');
    $this->add_single_action('cron_generate_title');
    $this->add_single_action('cron_generate_article');

    $this->is_test_exec = $_is_test_exec;
  }

  ///// 自動実行関数
  ///// 10分ごとに実行されるメイン関数
  ///// 実行対象のfeedsを抽出し、次のcronを登録する
  function run($calliope_feed_id = null)
  {
    CalliopeLogger::log('calliope_scheduler.........start');
    global $wpdb;
    $cfs_id = null;

    // ユーザー登録が終わってない or (クーポンコード未入力 or クーポンコード有効期限切れ) or サブスク未登録 の場合は実行しない
    if (!is_verified()) {
      CalliopeLogger::log('calliope_scheduler.........done(プラグイン使用不可の状態です)');
      return;
    };

    // メイン設定の「自動投稿を実行する」がOFFの場合 return
    $calliope_setting = $wpdb->get_row("SELECT * FROM " . CalliopeConfig::TABLE_CALLIOPE_SETTING . " WHERE id = 1");
    if ($this->is_test_exec == false && $calliope_setting->is_cron_exec == false) {
      CalliopeLogger::log('calliope_scheduler.........done(自動実行がOFFなので実行しません)');
      return;
    }

    if ($this->is_test_exec == true) {
      CalliopeLogger::log('calliope_scheduler.........テスト実行を開始します');

      // テスト実行
      $calliope_feed = $wpdb->get_row("SELECT * FROM " . CalliopeConfig::TABLE_CALLIOPE_FEEDS . " WHERE id = " . $calliope_feed_id);

      // 実行
      $cfs_id = $this->run_exec($calliope_feed);

    } else {
      // サブスクの投稿記事制限にかかっている場合はスキップ
      if(!is_verify_post_count()){
        CalliopeLogger::log('calliope_scheduler.........done(記事投稿数の制限です)');
        return;
      }

      // 通常の実行
      $calliope_feeds = $wpdb->get_results("SELECT * FROM " . CalliopeConfig::TABLE_CALLIOPE_FEEDS);

      foreach ($calliope_feeds as $calliope_feed) {
        // フィードのcron設定が「実行しない」であればcontinue
        if ($calliope_feed->cron_interval == 0) continue;

        $now = strtotime(current_time('Y-m-d H:i:s'));
        $last_cron_executed = $calliope_feed->last_cron_executed != null ? strtotime($calliope_feed->last_cron_executed) : strtotime('0000-00-00 00:00:00');
        $non_exec_interval = $now - $last_cron_executed;

        // cron実行間隔に到達していない場合continue
        if ($non_exec_interval < $calliope_feed->cron_interval) continue;

        // 実行
        $this->run_exec($calliope_feed);
      }
    }

    CalliopeLogger::log('calliope_scheduler.........done');

    // テスト実行のステータス表示用にIDを返却する
    return $cfs_id;
  }

  function run_exec($calliope_feed)
  {
    CalliopeLogger::log('$calliope_feed id.........' . $calliope_feed->id);

    global $wpdb;

    // calliope_feeds_scheduleにスケジュールを登録
    DBoperation::db_insert(
      CalliopeConfig::TABLE_CALLIOPE_FEEDS_SCHEDULE,
      [
        'status' => 'start',
        'calliope_feeds_id' => $calliope_feed->id
      ]
    );

    $id = $wpdb->insert_id;
    $uniqid = $this->get_unique_id();

    // 次のcronの登録
    $this->add_single_event('cron_generate_related_keyword', 5, ['uid' => $uniqid, 'id' => $id, 'is_test_exec' => $this->is_test_exec]);

    // テスト実行のときは最終実行時間を更新しない
    if ($this->is_test_exec == false) {
      // 最終実行時間を更新する
      $now = current_time('Y-m-d H:i:s');
      $wpdb->update(
        CalliopeConfig::TABLE_CALLIOPE_FEEDS,
        ['last_cron_executed' => $now],
        ['id' => $calliope_feed->id]
      );
    }

    // テスト実行のステータス表示用にIDを返却する
    return $id;
  }

  //////// 第1段階 : 関連キーワードの抽出
  function cron_generate_related_keyword($args)
  {
    $args = json_decode($args, true);
    $id = $args['id'];
    $uid = $args['uid'];
    $is_test_exec = $args['is_test_exec'];

    CalliopeLogger::log('cron_generate_related_keyword.........start[' . $uid . ']');

    $calliope_feeds_schedule = $this->get_calliope_feeds_schedule($id);
    $calliope_feed = $this->get_calliope_feed($calliope_feeds_schedule->calliope_feeds_id);

    // 関連キーワードを生成する
    $autoGenerator = $this->autoGeneratorConstruct($calliope_feed);
    $autoGenerator->generate_related_keyword();

    // スケジュール管理テーブルに一次保存
    DBoperation::db_update(
      CalliopeConfig::TABLE_CALLIOPE_FEEDS_SCHEDULE,
      [
        'status' => 'related_keyword',
        'related_keyword' => $autoGenerator->related_keyword
      ],
      ['id' => $id]
    );

    // 次のcronを登録
    $this->add_single_event('cron_generate_regulation', 5, ['uid' => $uid, 'id' => $id, 'is_test_exec' => $is_test_exec]);

    CalliopeLogger::log('cron_generate_related_keyword.........done[' . $uid . ']');
  }

  //////// 第1段階 : 関連キーワードの抽出
  function cron_generate_regulation($args)
  {
    $args = json_decode($args, true);
    $id = $args['id'];
    $uid = $args['uid'];
    $is_test_exec = $args['is_test_exec'];

    CalliopeLogger::log('cron_generate_regulation.........start[' . $uid . ']');

    $calliope_feeds_schedule = $this->get_calliope_feeds_schedule($id);
    $calliope_feed = $this->get_calliope_feed($calliope_feeds_schedule->calliope_feeds_id);

    // 関連キーワードを生成する
    $autoGenerator = $this->autoGeneratorConstruct($calliope_feed);
    $autoGenerator = $this->autoGeneratorInitialize($autoGenerator, $calliope_feeds_schedule);
    $autoGenerator->generate_regulation();

    // スケジュール管理テーブルに一次保存
    DBoperation::db_update(
      CalliopeConfig::TABLE_CALLIOPE_FEEDS_SCHEDULE,
      [
        'status' => 'regulation',
        'regulation' => $autoGenerator->regulation
      ],
      ['id' => $id]
    );

    // 次のcronを登録
    $this->add_single_event('cron_generate_indexes', 5, ['uid' => $uid, 'id' => $id, 'is_test_exec' => $is_test_exec]);

    CalliopeLogger::log('cron_generate_regulation.........done[' . $uid . ']');
  }

  //////// 第2段階 : 目次生成
  function cron_generate_indexes($args)
  {
    $args = json_decode($args, true);
    $id = $args['id'];
    $uid = $args['uid'];
    $is_test_exec = $args['is_test_exec'];

    CalliopeLogger::log('cron_generate_indexes.........start[' . $uid . ']');

    $calliope_feeds_schedule = $this->get_calliope_feeds_schedule($id);
    $calliope_feed = $this->get_calliope_feed($calliope_feeds_schedule->calliope_feeds_id);

    // 関連キーワードを生成
    $autoGenerator = $this->autoGeneratorConstruct($calliope_feed);
    $autoGenerator = $this->autoGeneratorInitialize($autoGenerator, $calliope_feeds_schedule);
    $autoGenerator->generate_indexes();

    // スケジュール管理テーブルに一次保存
    DBoperation::db_update(
      CalliopeConfig::TABLE_CALLIOPE_FEEDS_SCHEDULE,
      [
        'status' => 'index',
        'index_lists' => json_encode($autoGenerator->index_lists, JSON_UNESCAPED_UNICODE),
        'index_lists_str' => $autoGenerator->index_lists_str,
      ],
      ['id' => $id]
    );

    // 次のcronを登録
    // 後続の処理はindex_listがあれば並列処理が可能なので、一気にcronを登録しておく
    // titleが最後の処理になるので60秒後に実行
    $this->add_single_event('cron_generate_contents', 5, ['uid' => $uid, 'id' => $id, 'index' => 1, 'is_test_exec' => $is_test_exec]);
    $this->add_single_event('cron_generate_contents', 5, ['uid' => $uid, 'id' => $id, 'index' => 2, 'is_test_exec' => $is_test_exec]);
    $this->add_single_event('cron_generate_contents', 5, ['uid' => $uid, 'id' => $id, 'index' => 3, 'is_test_exec' => $is_test_exec]);
    $this->add_single_event('cron_generate_contents', 5, ['uid' => $uid, 'id' => $id, 'index' => 4, 'is_test_exec' => $is_test_exec]);
    $this->add_single_event('cron_generate_contents', 5, ['uid' => $uid, 'id' => $id, 'index' => 5, 'is_test_exec' => $is_test_exec]);
    $this->add_single_event('cron_generate_summary', 5, ['uid' => $uid, 'id' => $id, 'is_test_exec' => $is_test_exec]);
    $this->add_single_event('cron_generate_lead_text', 5, ['uid' => $uid, 'id' => $id, 'is_test_exec' => $is_test_exec]);
    $this->add_single_event('cron_generate_title', 60, ['uid' => $uid, 'id' => $id, 'is_test_exec' => $is_test_exec]);

    CalliopeLogger::log('cron_generate_indexes.........done');
  }

  //////// 第3段階 : 本文生成
  function cron_generate_contents($args)
  {
    $args = json_decode($args, true);
    $id = $args['id'];
    $uid = $args['uid'];
    $index = $args['index'];
    $is_test_exec = $args['is_test_exec'];

    CalliopeLogger::log('cron_generate_contents.........start[' . $uid . ']');

    $calliope_feeds_schedule = $this->get_calliope_feeds_schedule($id);
    $calliope_feed = $this->get_calliope_feed($calliope_feeds_schedule->calliope_feeds_id);

    // 本文の生成
    $autoGenerator = $this->autoGeneratorConstruct($calliope_feed);
    $autoGenerator = $this->autoGeneratorInitialize($autoGenerator, $calliope_feeds_schedule);
    $autoGenerator->generate_contents($index - 1);

    $_index_var = 'index' . $index;
    $_contents_var = 'contents' . $index;

    DBoperation::db_update(
      CalliopeConfig::TABLE_CALLIOPE_FEEDS_SCHEDULE,
      [
        'status' => $_contents_var,
        $_index_var => $autoGenerator->{$_index_var},
        $_contents_var => $autoGenerator->{$_contents_var},
      ],
      ['id' => $id]
    );

    CalliopeLogger::log('cron_generate_contents.........done[' . $uid . ']');
  }

  //////// 第4段階 : 要約生成
  function cron_generate_summary($args)
  {
    $args = json_decode($args, true);
    $id = $args['id'];
    $uid = $args['uid'];
    $is_test_exec = $args['is_test_exec'];

    CalliopeLogger::log('cron_generate_summary.........start[' . $uid . ']');

    $calliope_feeds_schedule = $this->get_calliope_feeds_schedule($id);
    $calliope_feed = $this->get_calliope_feed($calliope_feeds_schedule->calliope_feeds_id);

    // 要約の生成
    $autoGenerator = $this->autoGeneratorConstruct($calliope_feed);
    $autoGenerator = $this->autoGeneratorInitialize($autoGenerator, $calliope_feeds_schedule);
    $autoGenerator->generate_summary();

    DBoperation::db_update(
      CalliopeConfig::TABLE_CALLIOPE_FEEDS_SCHEDULE,
      [
        'status' => 'summary',
        'summary' => $autoGenerator->summary,
      ],
      ['id' => $id]
    );

    CalliopeLogger::log('cron_generate_summary.........done[' . $uid . ']');
  }

  //////// 第5段階 : リード文生成
  function cron_generate_lead_text($args)
  {
    $args = json_decode($args, true);
    $id = $args['id'];
    $uid = $args['uid'];
    $is_test_exec = $args['is_test_exec'];

    CalliopeLogger::log('cron_generate_lead_text.........start[' . $uid . ']');

    $calliope_feeds_schedule = $this->get_calliope_feeds_schedule($id);
    $calliope_feed = $this->get_calliope_feed($calliope_feeds_schedule->calliope_feeds_id);

    // リード文生成
    $autoGenerator = $this->autoGeneratorConstruct($calliope_feed);
    $autoGenerator = $this->autoGeneratorInitialize($autoGenerator, $calliope_feeds_schedule);
    $autoGenerator->generate_lead_text();

    DBoperation::db_update(
      CalliopeConfig::TABLE_CALLIOPE_FEEDS_SCHEDULE,
      [
        'status' => 'lead_text',
        'lead_text' => $autoGenerator->lead_text,
      ],
      ['id' => $id]
    );

    CalliopeLogger::log('cron_generate_lead_text.........done[' . $uid . ']');
  }

  //////// 第6段階 : タイトル生成
  function cron_generate_title($args)
  {
    $args = json_decode($args, true);
    $id = $args['id'];
    $uid = $args['uid'];
    $is_test_exec = $args['is_test_exec'];

    CalliopeLogger::log('cron_generate_title.........start[' . $uid . ']');

    $calliope_feeds_schedule = $this->get_calliope_feeds_schedule($id);
    $calliope_feed = $this->get_calliope_feed($calliope_feeds_schedule->calliope_feeds_id);

    // タイトル生成
    $autoGenerator = $this->autoGeneratorConstruct($calliope_feed);
    $autoGenerator = $this->autoGeneratorInitialize($autoGenerator, $calliope_feeds_schedule);
    $autoGenerator->generate_title();

    DBoperation::db_update(
      CalliopeConfig::TABLE_CALLIOPE_FEEDS_SCHEDULE,
      [
        'status' => 'title',
        'title' => $autoGenerator->title,
      ],
      ['id' => $id]
    );

    $this->add_single_event('cron_generate_article', 60, ['uid' => $uid, 'id' => $id, 'is_test_exec' => $is_test_exec]);

    CalliopeLogger::log('cron_generate_title.........done[' . $uid . ']');
  }

  //////// 記事の生成
  function cron_generate_article($args)
  {
    $args = json_decode($args, true);
    $id = $args['id'];
    $uid = $args['uid'];
    $is_test_exec = $args['is_test_exec'];

    CalliopeLogger::log('cron_generate_article.........start[' . $uid . ']');

    $calliope_feeds_schedule = $this->get_calliope_feeds_schedule($id);
    $calliope_feed = $this->get_calliope_feed($calliope_feeds_schedule->calliope_feeds_id);

    // autoGeneratorを初期化
    $autoGenerator = $this->autoGeneratorConstruct($calliope_feed);
    $autoGenerator = $this->autoGeneratorInitialize($autoGenerator, $calliope_feeds_schedule);

    // 記事を取得
    list($title, $contents) = $autoGenerator->generate_article();

    // カテゴリをfeedから取得する
    $categories = json_decode($calliope_feed->category);

    // タグは名称をカンマ区切りで連結した文字列で渡す必要があるため整形する
    $tags = json_decode($calliope_feed->tag);
    $_tags = [];
    $_tas = get_tags(['hide_empty' => false]);
    foreach ($tags as $value) {
      foreach ($_tas as $_ta) {
        if ($_ta->term_id == $value) {
          $_tags[] = $_ta->name;
        }
      }
    }
    $_tags = implode(',', $_tags);

    // テスト実行の場合は「下書き」で投稿する
    $status = $is_test_exec ? CalliopeConfig::CALLIOPE_POST_STATUS_VAL[0] : CalliopeConfig::CALLIOPE_POST_STATUS_VAL[$calliope_feed->status];

    // 投稿の生成
    $post = [
      'post_content' => $contents,
      'post_title'   => $title,
      'post_name'    => $title,
      'post_status'  => $status,
      'post_type'    => $calliope_feed->post_type,
      'post_category' => $categories,
      'tags_input'    => $_tags,
    ];
    $post_id = wp_insert_post($post);

    // パーマリンクの生成
    $calliope_feeds_schedule = $this->get_calliope_feeds_schedule($id);
    $calliope_feed = $this->get_calliope_feed($calliope_feeds_schedule->calliope_feeds_id);
    $autoGenerator = $this->autoGeneratorConstruct($calliope_feed);
    $autoGenerator = $this->autoGeneratorInitialize($autoGenerator, $calliope_feeds_schedule);
    $permalink = $autoGenerator->generate_permalink();

    // 投稿を更新
    $update_post = [
      'ID'        => $post_id,
      'post_name' => $permalink.'-'. $post_id,
    ];
    wp_update_post($update_post);

    // ステータスと記事IDの更新
    DBoperation::db_update(
      CalliopeConfig::TABLE_CALLIOPE_FEEDS_SCHEDULE,
      [
        'post_id' => $post_id,
        'status' => 'done',
      ],
      ['id' => $id]
    );

    CalliopeLogger::log('cron_generate_article.........done');
  }

  function autoGeneratorConstruct($calliope_feed)
  {
    return new AutoGenerator(
      $calliope_feed->keyword,
      $calliope_feed->related_keywords,
      $calliope_feed->custom_prompt,
      $calliope_feed->reference_urls,
      $calliope_feed->index_count,
      $calliope_feed->is_view_lead_text,
      $calliope_feed->is_view_index_ad,
      $calliope_feed->is_view_summary,
      $calliope_feed->is_view_last_ad,
      $calliope_feed->embedded_html
    );
  }

  ///// AutoGeneratorを外部から初期化する
  function autoGeneratorInitialize($autoGenerator, $calliope_feeds_schedule)
  {
    $autoGenerator->__init(
      $calliope_feeds_schedule->related_keyword,
      $calliope_feeds_schedule->regulation,
      json_decode($calliope_feeds_schedule->index_lists),
      $calliope_feeds_schedule->index_lists_str,
      $calliope_feeds_schedule->index1,
      $calliope_feeds_schedule->contents1,
      $calliope_feeds_schedule->index2,
      $calliope_feeds_schedule->contents2,
      $calliope_feeds_schedule->index3,
      $calliope_feeds_schedule->contents3,
      $calliope_feeds_schedule->index4,
      $calliope_feeds_schedule->contents4,
      $calliope_feeds_schedule->index5,
      $calliope_feeds_schedule->contents5,
      $calliope_feeds_schedule->summary,
      $calliope_feeds_schedule->lead_text,
      $calliope_feeds_schedule->title
    );

    return $autoGenerator;
  }

  ///// actionを登録する
  function add_single_action($action_name)
  {
    add_action($this->get_cron_slug($action_name), [$this, $action_name]);
  }

  ///// 一度きりのイベントを登録する
  function add_single_event($action_name, $time, $args)
  {
    $args = json_encode($args);
    $schedule_time = time() + $time;

    wp_schedule_single_event($schedule_time, $this->get_cron_slug($action_name), [$args]);
  }

  function get_cron_slug($action_name)
  {
    return 'calliope_cron_scheduler' . "_" . $action_name;
  }

  function get_unique_id()
  {
    return substr(str_shuffle('1234567890abcdefghijklmnopqrstuvwxyz'), 0, 15);
  }

  function get_calliope_feed($id)
  {
    global $wpdb;
    return $wpdb->get_row("SELECT * FROM " . CalliopeConfig::TABLE_CALLIOPE_FEEDS . " WHERE id = " . $id);
  }

  function get_calliope_feeds_schedule($id)
  {
    global $wpdb;
    return $wpdb->get_row("SELECT * FROM " . CalliopeConfig::TABLE_CALLIOPE_FEEDS_SCHEDULE . " WHERE id = " . $id);
  }
}
