<?php

require_once(get_plugin_root() . '/api/stripe.php');
require_once(get_plugin_root() . '/api/user_certification.php');

function get_plugin_root()
{
  return dirname(__FILE__);
}

function get_plugin_root_uri($file)
{
  return plugins_url($file, __FILE__);
}

function get_plugin_version()
{
  $plugin_data = get_plugin_data(get_plugin_root() . '/calliope.php');
  return $plugin_data['Version'];
}

function get_accessed_url_host()
{
  return (empty($_SERVER['HTTPS']) ? 'http://' : 'https://') . $_SERVER['HTTP_HOST'];
}

function get_accessed_url()
{
  return (empty($_SERVER['HTTPS']) ? 'http://' : 'https://') . $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI'];
}

function get_post_value($key)
{
  if (isset($_POST[$key])) return $_POST[$key];
  return null;
}

function get_get_value($key)
{
  if (isset($_GET[$key])) return $_GET[$key];
  return null;
}

function request_post($api_url, $data){
  $headers = ['Content-Type' => 'application/json',];
  $response = wp_remote_post($api_url, [
    'method' => 'POST',
    'headers' => $headers,
    'body' => json_encode($data),
  ]);

  return $response;
}

function get_object_value($obj, $key){
  if(!is_object($obj) || empty($obj)){
    return '';
  }

  if(property_exists($obj, $key)){
    return $obj->$key;
  }else{
    return '';
  }
}

function get_body($response)
{
  return json_decode(wp_remote_retrieve_body($response));
}

function return_response($code, $message_code, $message = '', $data = [])
{
  return [
    'code' => $code,
    'message_code' => $message_code,
    'message' => $message,
    'data' => $data
  ];
}

function get_endpoint($url)
{
  return 'https://us-central1-calliope-89149.cloudfunctions.net/' . $url;

  // if(CalliopeConfig::TEST_MODE){
  //   return 'http://127.0.0.1:5001/calliope-89149/us-central1/'.$url;
  // }else{
  //   return 'https://us-central1-calliope-89149.cloudfunctions.net/' . $url;
  // }
}

function check_is_user_created()
{
  global $wpdb;

  $calliope_user = $wpdb->get_row("SELECT * FROM " . CalliopeConfig::TABLE_CALLIOPE_USER . " WHERE id = 1");

  if (empty($calliope_user)) return false;

  return $calliope_user->is_user_created;
}

function check_is_coupon_code_registration()
{
  global $wpdb;

  $calliope_user = $wpdb->get_row("SELECT * FROM " . CalliopeConfig::TABLE_CALLIOPE_USER . " WHERE id = 1");

  if (empty($calliope_user)) return false;

  $uc = new UserCertification();
  $result = $uc->get_user($calliope_user->uid);

  if ($result['code'] == ApiBase::SUCCESS) {
    if(!empty($result['data']->user->couponCode)){
      return true;
    }else{
      return false;
    }
  }

  return false;
}

function redirect_menu_page()
{
  if (check_is_user_created()) {
    echo '
        <script type="text/javascript">

        setTimeout("redirect()", 0);

        function redirect() {
            location.href="' . admin_url('admin.php?page=calliope_menu') . '";
        }

        </script>';
    exit;
  }
}

function redirect_user_create_page()
{
  if (check_is_user_created()) {
    echo '
        <script type="text/javascript">

        setTimeout("redirect()", 0);

        function redirect() {
            location.href="' . admin_url('admin.php?page=calliope_user_create') . '";
        }

        </script>';
    exit;
  }
}

function redirect_subsuc_purchase_page()
{
  if (check_is_user_created()) {
    echo '
        <script type="text/javascript">

        setTimeout("redirect()", 0);

        function redirect() {
            location.href="' . admin_url('admin.php?page=calliope_subsc_purchase') . '";
        }

        </script>';
    exit;
  }
}

function redirect_subsuc_or_coupon_page()
{
  if (check_is_user_created()) {
    echo '
        <script type="text/javascript">

        setTimeout("redirect()", 0);

        function redirect() {
            location.href="' . admin_url('admin.php?page=calliope_subsuc_or_coupon') . '";
        }

        </script>';
    exit;
  }
}

function get_lists_anser($result)
{
  $separated_result = array_separator("\n", $result);
  if (count($separated_result) == 1) {
    $result = preg_replace('/(\d{1,}\.?\s?.+?\s)/', "$1", $result);
    $separated_result = array_separator("", $result);
  }
  $anser_arr = array();
  foreach ((array)$separated_result as $res) {
    $res = preg_replace('/\[gpt.*?\]/i', '', $res);
    $res = preg_replace('/\[\/gpt\]/i', '', $res);
    $res = preg_replace('/^\d{1,}\.?\s?/', '', $res);
    if (!$res) continue;
    $anser_arr[] = $res;
  }
  return $anser_arr;
}

function array_separator($delimita, $string)
{
  if (!$string or is_array($string)) {
    return;
  }
  $array = explode($delimita, $string);
  $array = array_map("trim", $array);
  return array_values(array_filter($array, "strlen"));
}

function is_verified(){
  global $wpdb;

  // ユーザー未登録の場合はfalse
  if(!check_is_user_created()){
    return false;
  }

  // ユーザーのメールアドレス
  $calliope_user = $wpdb->get_row("SELECT * FROM " . CalliopeConfig::TABLE_CALLIOPE_USER . " WHERE id = 1");
  $email = $calliope_user->mail;

  // 有効期限を取得する
  $uc = new UserCertification();
  $is_expire_date_verified = $uc->is_expire_date_verified();

  // サブスクの登録状況を確認する
  $stripe = new Stripe();
  $is_subsc_verfied = false;
  $s_result = $stripe->check_subscription_status($email);

  if ($s_result['code'] == ApiBase::SUCCESS) {
    if ($s_result['data']->status == "active") {
      $is_subsc_verfied = true;
    }
  }

  return $is_subsc_verfied || (check_is_coupon_code_registration() && $is_expire_date_verified);
}

// 有効記事数を取得する
function is_verify_post_count(){
  // 記事生成制限を取得する
  $post_limit = get_post_limit();

  // 生成した有効記事数を取得する
  $monthly_active_post_count = get_monthly_active_post_count();

  if(!empty($post_limit)){
    return $post_limit > $monthly_active_post_count;
  }else{
    return true;
  }
}

function get_post_limit(){
  global $wpdb;

  $post_limit = '';

  // ユーザーのメールアドレス
  $calliope_user = $wpdb->get_row("SELECT * FROM " . CalliopeConfig::TABLE_CALLIOPE_USER . " WHERE id = 1");

  if(empty($calliope_user)){
    return $post_limit;
  }

  $email = $calliope_user->mail;

  $stripe = new Stripe();
  $s_result = $stripe->check_subscription_status($email);

  if ($s_result['code'] == ApiBase::SUCCESS) {
    if ($s_result['data']->status == "active") {
      $product = $s_result['data']->product;

      if ($product->metadata->post_limit != 'no-limit') {
        $post_limit = $product->metadata->post_limit;
      }
    }
  }

  return $post_limit;
}

function get_monthly_active_post_count(){
  global $wpdb;

  $monthly_active_post_count = 0;

  // 今月初日
  $first_date = date("Y-m-01");

  $calliope_feeds_schedules = $wpdb->get_results(
    "SELECT * FROM " . CalliopeConfig::TABLE_CALLIOPE_FEEDS_SCHEDULE . "
    WHERE status = 'done'
    and created_at >= '" . $first_date . "'"
  );
  $posted_ids = [];
  foreach ($calliope_feeds_schedules as $key => $calliope_feeds_schedule) {
    $posted_ids[] = $calliope_feeds_schedule->post_id;
  }

  if (!empty($posted_ids)) {
    $active_posts = $wpdb->get_results(
      "SELECT * FROM wp_posts
       WHERE id in (" . implode(',', $posted_ids) . ")
       and post_type != 'revision'
       and post_type != 'page'
       and post_status != 'trash'"
    );
    $monthly_active_post_count = count($active_posts);
  }

  return $monthly_active_post_count;
}

function get_all_period_active_post_count()
{
  global $wpdb;

  $all_period_active_post_count = 0;

  $calliope_feeds_schedules = $wpdb->get_results(
    "SELECT * FROM " . CalliopeConfig::TABLE_CALLIOPE_FEEDS_SCHEDULE . "
    WHERE status = 'done'"
  );
  $posted_ids = [];
  foreach ($calliope_feeds_schedules as $key => $calliope_feeds_schedule) {
    $posted_ids[] = $calliope_feeds_schedule->post_id;
  }

  if (!empty($posted_ids)) {
    $active_posts = $wpdb->get_results(
      "SELECT * FROM wp_posts
       WHERE id in (" . implode(',', $posted_ids) . ")
       and post_type != 'revision'
       and post_type != 'page'
       and post_status != 'trash'"
    );
    $all_period_active_post_count = count($active_posts);
  }

  return $all_period_active_post_count;
}

function is_test_mode(){
  global $wpdb;

  $calliope_user = $wpdb->get_row("SELECT * FROM " . CalliopeConfig::TABLE_CALLIOPE_USER . " WHERE id = 1");

  $md = new Mode();
  $result = $md->get_mode($calliope_user->uid);

  $mode = Mode::IS_NOT_TEST;

  if ($result['code'] == ApiBase::SUCCESS) {
    if (!empty($result['data']->mode)) {
      $mode = $result['data']->mode;
    }
  }

  return $mode == Mode::IS_TEST ? true : false;
}